@@ -7,11 +7,11 @@ package org.h2.command.ddl;
 
 import java.util.ArrayList;
 import java.util.HashSet;
-
 import org.h2.api.ErrorCode;
 import org.h2.command.CommandInterface;
 import org.h2.command.dml.Insert;
 import org.h2.command.dml.Query;
+import org.h2.engine.Constants;
 import org.h2.engine.Database;
 import org.h2.engine.DbObject;
 import org.h2.engine.Session;
@@ -140,6 +140,11 @@ public class CreateTable extends SchemaCommand {
             if (c.isAutoIncrement()) {
                 int objId = getObjectId();
                 c.convertAutoIncrementToSequence(session, getSchema(), objId, data.temporary);
+                if (!Constants.CLUSTERING_DISABLED
+                        .equals(session.getDatabase().getCluster())) {
+                    throw DbException.getUnsupportedException(
+                            "CLUSTERING && auto-increment columns");
+                }
             }
             Sequence seq = c.getSequence();
             if (seq != null) {
