@@ -22,9 +22,9 @@ import java.util.concurrent.ConcurrentHashMap;
 import org.h2.compress.CompressDeflate;
 import org.h2.compress.CompressLZF;
 import org.h2.compress.Compressor;
+import org.h2.mvstore.Page.PageChildren;
 import org.h2.mvstore.cache.CacheLongKeyLIRS;
 import org.h2.mvstore.type.StringDataType;
-import org.h2.mvstore.Page.PageChildren;
 import org.h2.util.MathUtils;
 import org.h2.util.New;
 
@@ -49,19 +49,16 @@ MVStore:
     even for a small heap, and so that chunks
     are still relatively big on average
 - make sure serialization / deserialization errors don't corrupt the file
-- FileStore: don't open and close when set using MVStore.Builder.fileStore
 - test and possibly improve compact operation (for large dbs)
-- is data kept in the stream store if the transaction is not committed?
 - automated 'kill process' and 'power failure' test
-- compact: avoid processing pages using a counting bloom filter
 - defragment (re-creating maps, specially those with small pages)
 - store number of write operations per page (maybe defragment
     if much different than count)
 - r-tree: nearest neighbor search
 - use a small object value cache (StringCache), test on Android
     for default serialization
-- MVStoreTool.dump: dump values (using a callback)
-- close the file on out of memory or disk write error (out of disk space or so)
+- MVStoreTool.dump should dump the data if possible;
+    possibly using a callback for serialization
 - implement a sharded map (in one store, multiple stores)
     to support concurrent updates and writes, and very large maps
 - to save space when persisting very small transactions,
@@ -72,8 +69,6 @@ MVStore:
 - remove features that are not really needed; simplify the code
     possibly using a separate layer or tools
     (retainVersion?)
-- MVStoreTool.dump should dump the data if possible;
-    possibly using a callback for serialization
 - optional pluggable checksum mechanism (per page), which
     requires that everything is a page (including headers)
 - rename "store" to "save", as "store" is used in "storeVersion"
@@ -81,7 +76,6 @@ MVStore:
 - temporary file storage
 - simple rollback method (rollback to last committed version)
 - MVMap to implement SortedMap, then NavigableMap
-- Test with OSGi
 - storage that splits database into multiple files,
     to speed up compact and allow using trim
     (by truncating / deleting empty files)
@@ -98,7 +92,6 @@ MVStore:
 - write a LSM-tree (log structured merge tree) utility on top of the MVStore
     with blind writes and/or a bloom filter that
     internally uses regular maps and merge sort
-- LIRS cache: maybe remove 'mask' field, and dynamically grow the arrays
 - chunk metadata: maybe split into static and variable,
     or use a small page size for metadata
 - data type "string": maybe use prefix compression for keys
@@ -122,7 +115,6 @@ MVStore:
 - rollback of removeMap should restore the data -
     which has big consequences, as the metadata map
     would probably need references to the root nodes of all maps
-- combine MVMap and MVMapConcurrent
 
 */
 
@@ -208,7 +200,7 @@ public class MVStore {
     private final ConcurrentHashMap<Integer, MVMap<?, ?>> maps =
             new ConcurrentHashMap<Integer, MVMap<?, ?>>();
 
-    private HashMap<String, Object> fileHeader = New.hashMap();
+    private HashMap<String, Object> storeHeader = New.hashMap();
 
     private WriteBuffer writeBuffer;
 
@@ -248,6 +240,13 @@ public class MVStore {
      * The time the store was created, in milliseconds since 1970.
      */
     private long creationTime;
+
+    /**
+     * How long to retain old, persisted chunks, in milliseconds. For larger or
+     * equal to zero, a chunk is never directly overwritten if unused, but
+     * instead, the unused field is set. If smaller zero, chunks are directly
+     * overwritten if unused.
+     */
     private int retentionTime;
 
     private long lastCommitTime;
@@ -278,6 +277,10 @@ public class MVStore {
 
     private IllegalStateException panicException;
 
+    private long lastTimeAbsolute;
+
+    private long lastFreeUnusedChunks;
+
     /**
      * Create and open the store.
      *
@@ -321,16 +324,15 @@ public class MVStore {
         o = config.get("cacheSize");
         int mb = o == null ? 16 : (Integer) o;
         if (mb > 0) {
-            int maxMemoryBytes = mb * 1024 * 1024;
-            int averageMemory = Math.max(10, pageSplitSize / 2);
-            int segmentCount = 16;
-            int stackMoveDistance = maxMemoryBytes / averageMemory * 2 / 100;
-            cache = new CacheLongKeyLIRS<Page>(
-                    maxMemoryBytes, averageMemory,
-                    segmentCount, stackMoveDistance);
-            cacheChunkRef = new CacheLongKeyLIRS<PageChildren>(
-                    maxMemoryBytes / 4, 20,
-                    segmentCount, stackMoveDistance);
+            CacheLongKeyLIRS.Config cc = new CacheLongKeyLIRS.Config();
+            cc.maxMemory = mb * 1024L * 1024L;
+            o = config.get("cacheConcurrency");
+            if (o != null) {
+                cc.segmentCount = (Integer) o;
+            }
+            cache = new CacheLongKeyLIRS<Page>(cc);
+            cc.maxMemory /= 4;
+            cacheChunkRef = new CacheLongKeyLIRS<PageChildren>(cc);
         }
         o = config.get("autoCommitBufferSize");
         int kb = o == null ? 1024 : (Integer) o;
@@ -346,15 +348,15 @@ public class MVStore {
                 fileStore.open(fileName, readOnly, encryptionKey);
             }
             if (fileStore.size() == 0) {
-                creationTime = getTime();
+                creationTime = getTimeAbsolute();
                 lastCommitTime = creationTime;
-                fileHeader.put("H", 2);
-                fileHeader.put("blockSize", BLOCK_SIZE);
-                fileHeader.put("format", FORMAT_WRITE);
-                fileHeader.put("created", creationTime);
-                writeFileHeader();
+                storeHeader.put("H", 2);
+                storeHeader.put("blockSize", BLOCK_SIZE);
+                storeHeader.put("format", FORMAT_WRITE);
+                storeHeader.put("created", creationTime);
+                writeStoreHeader();
             } else {
-                readFileHeader();
+                readStoreHeader();
             }
         } catch (IllegalStateException e) {
             panic(e);
@@ -363,7 +365,7 @@ public class MVStore {
                 Arrays.fill(encryptionKey, (char) 0);
             }
         }
-        lastCommitTime = getTime();
+        lastCommitTime = getTimeSinceCreation();
 
         // setAutoCommitDelay starts the thread, but only if
         // the parameter is different from the old value
@@ -529,13 +531,15 @@ public class MVStore {
     }
 
     private Chunk getChunkForVersion(long version) {
-        Chunk c = lastChunk;
-        while (true) {
-            if (c == null || c.version <= version) {
-                return c;
+        Chunk newest = null;
+        for (Chunk c : chunks.values()) {
+            if (c.version <= version) {
+                if (newest == null || c.id > newest.id) {
+                    newest = c;
+                }
             }
-            c = chunks.get(c.id - 1);
         }
+        return newest;
     }
 
     /**
@@ -554,11 +558,10 @@ public class MVStore {
         metaChanged = true;
     }
 
-    private synchronized void readFileHeader() {
-        boolean validHeader = false;
-        // we don't know yet which chunk and version are the newest
-        long newestVersion = -1;
-        long chunkBlock = -1;
+    private synchronized void readStoreHeader() {
+        Chunk newest = null;
+        boolean validStoreHeader = false;
+        // find out which chunk and version are the newest
         // read the first two blocks
         ByteBuffer fileHeaderBlocks = fileStore.readFully(0, 2 * BLOCK_SIZE);
         byte[] buff = new byte[BLOCK_SIZE];
@@ -587,23 +590,27 @@ public class MVStore {
                     continue;
                 }
                 long version = DataUtils.readHexLong(m, "version", 0);
-                if (version > newestVersion) {
-                    newestVersion = version;
-                    fileHeader.putAll(m);
-                    chunkBlock = DataUtils.readHexLong(m, "block", 0);
+                if (newest == null || version > newest.version) {
+                    validStoreHeader = true;
+                    storeHeader.putAll(m);
                     creationTime = DataUtils.readHexLong(m, "created", 0);
-                    validHeader = true;
+                    int chunkId = DataUtils.readHexInt(m, "chunk", 0);
+                    long block = DataUtils.readHexLong(m, "block", 0);
+                    Chunk test = readChunkHeaderAndFooter(block);
+                    if (test != null && test.id == chunkId) {
+                        newest = test;
+                    }
                 }
             } catch (Exception e) {
                 continue;
             }
         }
-        if (!validHeader) {
+        if (!validStoreHeader) {
             throw DataUtils.newIllegalStateException(
                     DataUtils.ERROR_FILE_CORRUPT,
                     "Store header is corrupt: {0}", fileStore);
         }
-        long format = DataUtils.readHexLong(fileHeader, "format", 1);
+        long format = DataUtils.readHexLong(storeHeader, "format", 1);
         if (format > FORMAT_WRITE && !fileStore.isReadOnly()) {
             throw DataUtils.newIllegalStateException(
                     DataUtils.ERROR_UNSUPPORTED_FORMAT,
@@ -612,7 +619,7 @@ public class MVStore {
                     "and the file was not opened in read-only mode",
                     format, FORMAT_WRITE);
         }
-        format = DataUtils.readHexLong(fileHeader, "formatRead", format);
+        format = DataUtils.readHexLong(storeHeader, "formatRead", format);
         if (format > FORMAT_READ) {
             throw DataUtils.newIllegalStateException(
                     DataUtils.ERROR_UNSUPPORTED_FORMAT,
@@ -636,60 +643,53 @@ public class MVStore {
             // the system time was set to the past:
             // we change the creation time
             creationTime = now;
-            fileHeader.put("created", creationTime);
+            storeHeader.put("created", creationTime);
         }
-
-        Chunk footer = readChunkFooter(fileStore.size());
-        if (footer != null) {
-            if (footer.version > newestVersion) {
-                newestVersion = footer.version;
-                chunkBlock = footer.block;
+        Chunk test = readChunkFooter(fileStore.size());
+        if (test != null) {
+            test = readChunkHeaderAndFooter(test.block);
+            if (test != null) {
+                if (newest == null || test.version > newest.version) {
+                    newest = test;
+                }
             }
         }
-        if (chunkBlock <= 0) {
+        if (newest == null) {
             // no chunk
             return;
         }
-
         // read the chunk header and footer,
         // and follow the chain of next chunks
-        lastChunk = null;
         while (true) {
-            Chunk header;
-            try {
-                header = readChunkHeader(chunkBlock);
-            } catch (Exception e) {
-                // invalid chunk header: ignore, but stop
-                break;
-            }
-            if (header.version < newestVersion) {
-                // we have reached the end
-                break;
-            }
-            footer = readChunkFooter((chunkBlock + header.len) * BLOCK_SIZE);
-            if (footer == null || footer.id != header.id) {
-                // invalid chunk footer, or the wrong one
-                break;
-            }
-            lastChunk = header;
-            newestVersion = header.version;
-            if (header.next == 0 ||
-                    header.next >= fileStore.size() / BLOCK_SIZE) {
+            if (newest.next == 0 ||
+                    newest.next >= fileStore.size() / BLOCK_SIZE) {
                 // no (valid) next
                 break;
             }
-            chunkBlock = header.next;
+            test = readChunkHeaderAndFooter(newest.next);
+            if (test == null || test.id <= newest.id) {
+                break;
+            }
+            newest = test;
         }
-        if (lastChunk == null) {
-            // no valid chunk
-            return;
+        setLastChunk(newest);
+        loadChunkMeta();
+        // read all chunk headers and footers within the retention time,
+        // to detect unwritten data after a power failure
+        verifyLastChunks();
+        // build the free space list
+        for (Chunk c : chunks.values()) {
+            if (c.pageCountLive == 0) {
+                // remove this chunk in the next save operation
+                registerFreePage(currentVersion, c.id, 0, 0);
+            }
+            long start = c.block * BLOCK_SIZE;
+            int length = c.len * BLOCK_SIZE;
+            fileStore.markUsed(start, length);
         }
-        lastMapId = lastChunk.mapId;
-        currentVersion = lastChunk.version;
-        setWriteVersion(currentVersion);
-        chunks.put(lastChunk.id, lastChunk);
-        meta.setRootPos(lastChunk.metaRootPos, -1);
+    }
 
+    private void loadChunkMeta() {
         // load the chunk metadata: we can load in any order,
         // because loading chunk metadata might recursively load another chunk
         for (Iterator<String> it = meta.keyIterator("chunk."); it.hasNext();) {
@@ -708,16 +708,81 @@ public class MVStore {
                 chunks.put(c.id, c);
             }
         }
-        // build the free space list
-        for (Chunk c : chunks.values()) {
-            if (c.pageCountLive == 0) {
-                // remove this chunk in the next save operation
-                registerFreePage(currentVersion, c.id, 0, 0);
-            }
-            long start = c.block * BLOCK_SIZE;
-            int length = c.len * BLOCK_SIZE;
-            fileStore.markUsed(start, length);
+    }
+
+    private void setLastChunk(Chunk last) {
+        lastChunk = last;
+        if (last == null) {
+            // no valid chunk
+            lastMapId = 0;
+            currentVersion = 0;
+            meta.setRootPos(0, -1);
+        } else {
+            lastMapId = last.mapId;
+            currentVersion = last.version;
+            chunks.put(last.id, last);
+            meta.setRootPos(last.metaRootPos, -1);
         }
+        setWriteVersion(currentVersion);
+    }
+
+    private void verifyLastChunks() {
+        long time = getTimeSinceCreation();
+        ArrayList<Integer> ids = new ArrayList<Integer>(chunks.keySet());
+        Collections.sort(ids);
+        int newestValidChunk = -1;
+        Chunk old = null;
+        for (Integer chunkId : ids) {
+            Chunk c = chunks.get(chunkId);
+            if (old != null && c.time < old.time) {
+                // old chunk (maybe leftover from a previous crash)
+                break;
+            }
+            old = c;
+            if (c.time + retentionTime < time) {
+                // old chunk, no need to verify
+                newestValidChunk = c.id;
+                continue;
+            }
+            Chunk test = readChunkHeaderAndFooter(c.block);
+            if (test == null || test.id != c.id) {
+                break;
+            }
+            newestValidChunk = chunkId;
+        }
+        Chunk newest = chunks.get(newestValidChunk);
+        if (newest != lastChunk) {
+            // to avoid re-using newer chunks later on, we could clear
+            // the headers and footers of those, but we might not know about all
+            // of them, so that could be incomplete - but we check that newer
+            // chunks are written after older chunks, so we are safe
+            rollbackTo(newest == null ? 0 : newest.version);
+        }
+    }
+
+    /**
+     * Read a chunk header and footer, and verify the stored data is consistent.
+     *
+     * @param block the block
+     * @return the chunk, or null if the header or footer don't match or are not
+     *         consistent
+     */
+    private Chunk readChunkHeaderAndFooter(long block) {
+        Chunk header;
+        try {
+            header = readChunkHeader(block);
+        } catch (Exception e) {
+            // invalid chunk header: ignore, but stop
+            return null;
+        }
+        if (header == null) {
+            return null;
+        }
+        Chunk footer = readChunkFooter((block + header.len) * BLOCK_SIZE);
+        if (footer == null || footer.id != header.id) {
+            return null;
+        }
+        return header;
     }
 
     /**
@@ -754,14 +819,14 @@ public class MVStore {
         return null;
     }
 
-    private void writeFileHeader() {
+    private void writeStoreHeader() {
         StringBuilder buff = new StringBuilder();
         if (lastChunk != null) {
-            fileHeader.put("block", lastChunk.block);
-            fileHeader.put("chunk", lastChunk.id);
-            fileHeader.put("version", lastChunk.version);
+            storeHeader.put("block", lastChunk.block);
+            storeHeader.put("chunk", lastChunk.id);
+            storeHeader.put("version", lastChunk.version);
         }
-        DataUtils.appendMap(buff, fileHeader);
+        DataUtils.appendMap(buff, storeHeader);
         byte[] bytes = buff.toString().getBytes(DataUtils.LATIN);
         int checksum = DataUtils.getFletcher32(bytes, bytes.length);
         DataUtils.appendMap(buff, "fletcher", checksum);
@@ -791,7 +856,8 @@ public class MVStore {
         if (closed) {
             return;
         }
-        if (fileStore != null && !fileStore.isReadOnly()) {
+        FileStore f = fileStore;
+        if (f != null && !f.isReadOnly()) {
             stopBackgroundThread();
             if (hasUnsavedChanges()) {
                 commitAndSave();
@@ -882,6 +948,7 @@ public class MVStore {
         int chunkId = DataUtils.getPageChunkId(pos);
         Chunk c = chunks.get(chunkId);
         if (c == null) {
+            checkOpen();
             if (!Thread.holdsLock(this)) {
                 // it could also be unsynchronized metadata
                 // access (if synchronization on this was forgotten)
@@ -909,7 +976,11 @@ public class MVStore {
         for (MVMap<?, ?> map : maps.values()) {
             map.setWriteVersion(version);
         }
-        meta.setWriteVersion(version);
+        MVMap<String, String> m = meta;
+        if (m == null) {
+            checkOpen();
+        }
+        m.setWriteVersion(version);
     }
 
     /**
@@ -986,12 +1057,19 @@ public class MVStore {
     }
 
     private long storeNowTry() {
-        freeUnusedChunks();
+        long time = getTimeSinceCreation();
+        int freeDelay = retentionTime / 10;
+        if (time >= lastFreeUnusedChunks + freeDelay) {
+            // set early in case it fails (out of memory or so)
+            lastFreeUnusedChunks = time;
+            freeUnusedChunks();
+            // set it here as well, to avoid calling it often if it was slow
+            lastFreeUnusedChunks = getTimeSinceCreation();
+        }
         int currentUnsavedPageCount = unsavedMemory;
         long storeVersion = currentStoreVersion;
         long version = ++currentVersion;
         setWriteVersion(version);
-        long time = getTime();
         lastCommitTime = time;
         retainChunk = null;
 
@@ -1008,9 +1086,19 @@ public class MVStore {
             time = Math.max(lastChunk.time, time);
         }
         int newChunkId = lastChunkId;
-        do {
+        while (true) {
             newChunkId = (newChunkId + 1) % Chunk.MAX_ID;
-        } while (chunks.containsKey(newChunkId));
+            Chunk old = chunks.get(newChunkId);
+            if (old == null) {
+                break;
+            }
+            if (old.block == Long.MAX_VALUE) {
+                IllegalStateException e = DataUtils.newIllegalStateException(
+                        DataUtils.ERROR_INTERNAL,
+                        "Last block not stored, possibly due to out-of-memory");
+                panic(e);
+            }
+        }
         Chunk c = new Chunk(newChunkId);
 
         c.pageCount = Integer.MAX_VALUE;
@@ -1123,28 +1211,28 @@ public class MVStore {
         write(filePos, buff.getBuffer());
         releaseWriteBuffer(buff);
 
-        // whether we need to write the file header
-        boolean needHeader = false;
+        // whether we need to write the store header
+        boolean writeStoreHeader = false;
         if (!storeAtEndOfFile) {
             if (lastChunk == null) {
-                needHeader = true;
+                writeStoreHeader = true;
             } else if (lastChunk.next != c.block) {
                 // the last prediction did not matched
-                needHeader = true;
+                writeStoreHeader = true;
             } else {
                 long headerVersion = DataUtils.readHexLong(
-                        fileHeader, "version", 0);
+                        storeHeader, "version", 0);
                 if (lastChunk.version - headerVersion > 20) {
                     // we write after at least 20 entries
-                    needHeader = true;
+                    writeStoreHeader = true;
                 } else {
-                    int chunkId = DataUtils.readHexInt(fileHeader, "chunk", 0);
+                    int chunkId = DataUtils.readHexInt(storeHeader, "chunk", 0);
                     while (true) {
                         Chunk old = chunks.get(chunkId);
                         if (old == null) {
                             // one of the chunks in between
                             // was removed
-                            needHeader = true;
+                            writeStoreHeader = true;
                             break;
                         }
                         if (chunkId == lastChunk.id) {
@@ -1157,14 +1245,13 @@ public class MVStore {
         }
 
         lastChunk = c;
-        if (needHeader) {
-            writeFileHeader();
+        if (writeStoreHeader) {
+            writeStoreHeader();
         }
         if (!storeAtEndOfFile) {
-            // may only shrink after the file header was written
+            // may only shrink after the store header was written
             shrinkFileIfPossible(1);
         }
-
         for (MVMap<?, ?> m : changed) {
             Page p = m.getRoot();
             if (p.getTotalCount() > 0) {
@@ -1185,12 +1272,12 @@ public class MVStore {
     }
 
     private synchronized void freeUnusedChunks() {
-        if (lastChunk == null) {
+        if (lastChunk == null || !reuseSpace) {
             return;
         }
         Set<Integer> referenced = collectReferencedChunks();
         ArrayList<Chunk> free = New.arrayList();
-        long time = getTime();
+        long time = getTimeSinceCreation();
         for (Chunk c : chunks.values()) {
             if (!referenced.contains(c.id)) {
                 free.add(c);
@@ -1229,27 +1316,45 @@ public class MVStore {
                 continue;
             }
             int mapId = DataUtils.parseHexInt(key.substring("root.".length()));
-            collectReferencedChunks(referenced, mapId, pos);
+            collectReferencedChunks(referenced, mapId, pos, 0);
         }
         long pos = lastChunk.metaRootPos;
-        collectReferencedChunks(referenced, 0, pos);
+        collectReferencedChunks(referenced, 0, pos, 0);
         readCount = fileStore.readCount - readCount;
         return referenced;
     }
 
-    private int collectReferencedChunks(Set<Integer> targetChunkSet, int mapId, long pos) {
-        targetChunkSet.add(DataUtils.getPageChunkId(pos));
+    private void collectReferencedChunks(Set<Integer> targetChunkSet,
+            int mapId, long pos, int level) {
+        int c = DataUtils.getPageChunkId(pos);
+        targetChunkSet.add(c);
         if (DataUtils.getPageType(pos) == DataUtils.PAGE_TYPE_LEAF) {
-            return 1;
+            return;
         }
         PageChildren refs = readPageChunkReferences(mapId, pos, -1);
-        int count = 0;
-        if (refs != null) {
-            for (long p : refs.children) {
-                count += collectReferencedChunks(targetChunkSet, mapId, p);
+        if (!refs.chunkList) {
+            Set<Integer> target = New.hashSet();
+            for (int i = 0; i < refs.children.length; i++) {
+                long p = refs.children[i];
+                collectReferencedChunks(target, mapId, p, level + 1);
+            }
+            // we don't need a reference to this chunk
+            target.remove(c);
+            long[] children = new long[target.size()];
+            int i = 0;
+            for (Integer p : target) {
+                children[i++] = DataUtils.getPagePos(p, 0, 0,
+                        DataUtils.PAGE_TYPE_LEAF);
+            }
+            refs.children = children;
+            refs.chunkList = true;
+            if (cacheChunkRef != null) {
+                cacheChunkRef.put(refs.pos, refs, refs.getMemory());
             }
         }
-        return count;
+        for (long p : refs.children) {
+            targetChunkSet.add(DataUtils.getPageChunkId(p));
+        }
     }
 
     private PageChildren readPageChunkReferences(int mapId, long pos, int parentChunk) {
@@ -1263,6 +1368,7 @@ public class MVStore {
             r = null;
         }
         if (r == null) {
+            // if possible, create it from the cached page
             if (cache != null) {
                 Page p = cache.get(pos);
                 if (p != null) {
@@ -1270,6 +1376,7 @@ public class MVStore {
                 }
             }
             if (r == null) {
+                // page was not cached: read the data
                 Chunk c = getChunk(pos);
                 long filePos = c.block * BLOCK_SIZE;
                 filePos += DataUtils.getPageOffset(pos);
@@ -1283,7 +1390,7 @@ public class MVStore {
             }
             r.removeDuplicateChunkReferences();
             if (cacheChunkRef != null) {
-                cacheChunkRef.put(pos, r);
+                cacheChunkRef.put(pos, r, r.getMemory());
             }
         }
         if (r.children.length == 0) {
@@ -1325,11 +1432,13 @@ public class MVStore {
     }
 
     private boolean canOverwriteChunk(Chunk c, long time) {
-        if (c.time + retentionTime > time) {
-            return false;
-        }
-        if (c.unused == 0 || c.unused + retentionTime / 2 > time) {
-            return false;
+        if (retentionTime >= 0) {
+            if (c.time + retentionTime > time) {
+                return false;
+            }
+            if (c.unused == 0 || c.unused + retentionTime / 2 > time) {
+                return false;
+            }
         }
         Chunk r = retainChunk;
         if (r != null && c.version > r.version) {
@@ -1338,8 +1447,21 @@ public class MVStore {
         return true;
     }
 
-    private long getTime() {
-        return System.currentTimeMillis() - creationTime;
+    private long getTimeSinceCreation() {
+        return Math.max(0, getTimeAbsolute() - creationTime);
+    }
+
+    private long getTimeAbsolute() {
+        long now = System.currentTimeMillis();
+        if (lastTimeAbsolute != 0 && now < lastTimeAbsolute) {
+            // time seems to have run backwards - this can happen
+            // when the system time is adjusted, for example
+            // on a leap second
+            now = lastTimeAbsolute;
+        } else {
+            lastTimeAbsolute = now;
+        }
+        return now;
     }
 
     /**
@@ -1349,8 +1471,7 @@ public class MVStore {
      *
      * @param storeVersion apply up to the given version
      */
-    private Set<Chunk> applyFreedSpace(long storeVersion) {
-        Set<Chunk> removedChunks = New.hashSet();
+    private void applyFreedSpace(long storeVersion) {
         while (true) {
             ArrayList<Chunk> modified = New.arrayList();
             Iterator<Entry<Long, HashMap<Integer, Chunk>>> it;
@@ -1373,20 +1494,12 @@ public class MVStore {
                     c.maxLenLive += f.maxLenLive;
                     c.pageCountLive += f.pageCountLive;
                     if (c.pageCountLive < 0 && c.pageCountLive > -MARKED_FREE) {
-                        throw DataUtils.newIllegalStateException(
-                                DataUtils.ERROR_INTERNAL,
-                                "Corrupt page count {0}", c.pageCountLive);
+                        // can happen after a rollback
+                        c.pageCountLive = 0;
                     }
                     if (c.maxLenLive < 0 && c.maxLenLive > -MARKED_FREE) {
-                        throw DataUtils.newIllegalStateException(
-                                DataUtils.ERROR_INTERNAL,
-                                "Corrupt max length {0}", c.maxLenLive);
-                    }
-                    if (c.pageCountLive <= 0 && c.maxLenLive > 0 ||
-                            c.maxLenLive <= 0 && c.pageCountLive > 0) {
-                        throw DataUtils.newIllegalStateException(
-                                DataUtils.ERROR_INTERNAL,
-                                "Corrupt max length {0}", c.maxLenLive);
+                        // can happen after a rollback
+                        c.maxLenLive = 0;
                     }
                     modified.add(c);
                 }
@@ -1399,7 +1512,6 @@ public class MVStore {
                 break;
             }
         }
-        return removedChunks;
     }
 
     /**
@@ -1409,6 +1521,9 @@ public class MVStore {
      * @param minPercent the minimum percentage to save
      */
     private void shrinkFileIfPossible(int minPercent) {
+        if (fileStore.isReadOnly()) {
+            return;
+        }
         long end = getFileLengthInUse();
         long fileSize = fileStore.size();
         if (end >= fileSize) {
@@ -1421,6 +1536,9 @@ public class MVStore {
         if (savedPercent < minPercent) {
             return;
         }
+        if (!closed) {
+            sync();
+        }
         fileStore.truncate(end);
     }
 
@@ -1521,14 +1639,14 @@ public class MVStore {
      */
     public synchronized boolean compactMoveChunks(int targetFillRate, long moveSize) {
         checkOpen();
-        if (lastChunk == null) {
+        if (lastChunk == null || !reuseSpace) {
             // nothing to do
             return false;
         }
         int oldRetentionTime = retentionTime;
         boolean oldReuse = reuseSpace;
         try {
-            retentionTime = 0;
+            retentionTime = -1;
             freeUnusedChunks();
             if (fileStore.getFillRate() > targetFillRate) {
                 return false;
@@ -1598,7 +1716,7 @@ public class MVStore {
             buff.position(0);
             c.writeChunkHeader(buff, chunkHeaderLen);
             buff.position(length - Chunk.FOOTER_LENGTH);
-            buff.put(lastChunk.getFooterBytes());
+            buff.put(c.getFooterBytes());
             buff.position(0);
             write(end, buff.getBuffer());
             releaseWriteBuffer(buff);
@@ -1609,7 +1727,6 @@ public class MVStore {
         // update the metadata (store at the end of the file)
         reuseSpace = false;
         commitAndSave();
-
         sync();
 
         // now re-use the empty space
@@ -1635,7 +1752,7 @@ public class MVStore {
             c.block = pos / BLOCK_SIZE;
             c.writeChunkHeader(buff, chunkHeaderLen);
             buff.position(length - Chunk.FOOTER_LENGTH);
-            buff.put(lastChunk.getFooterBytes());
+            buff.put(c.getFooterBytes());
             buff.position(0);
             write(pos, buff.getBuffer());
             releaseWriteBuffer(buff);
@@ -1654,7 +1771,11 @@ public class MVStore {
      * implementation calls FileChannel.force(true).
      */
     public void sync() {
-        fileStore.sync();
+        checkOpen();
+        FileStore f = fileStore;
+        if (f != null) {
+            f.sync();
+        }
     }
 
     /**
@@ -1676,6 +1797,9 @@ public class MVStore {
      * @return if a chunk was re-written
      */
     public boolean compact(int targetFillRate, int write) {
+        if (!reuseSpace) {
+            return false;
+        }
         synchronized (compactSync) {
             checkOpen();
             ArrayList<Chunk> old;
@@ -1700,7 +1824,7 @@ public class MVStore {
         long maxLengthSum = 0;
         long maxLengthLiveSum = 0;
 
-        long time = getTime();
+        long time = getTimeSinceCreation();
 
         for (Chunk c : chunks.values()) {
             // ignore young chunks, because we don't optimize those
@@ -2038,15 +2162,27 @@ public class MVStore {
         if (oldMeta == null) {
             return false;
         }
-        for (Iterator<String> it = oldMeta.keyIterator("chunk.");
-                it.hasNext();) {
-            String chunkKey = it.next();
-            if (!chunkKey.startsWith("chunk.")) {
-                break;
-            }
-            if (!meta.containsKey(chunkKey)) {
-                return false;
+        try {
+            for (Iterator<String> it = oldMeta.keyIterator("chunk.");
+                    it.hasNext();) {
+                String chunkKey = it.next();
+                if (!chunkKey.startsWith("chunk.")) {
+                    break;
+                }
+                if (!meta.containsKey(chunkKey)) {
+                    String s = oldMeta.get(chunkKey);
+                    Chunk c2 = Chunk.fromString(s);
+                    Chunk test = readChunkHeaderAndFooter(c2.block);
+                    if (test == null || test.id != c2.id) {
+                        return false;
+                    }
+                    // we store this chunk
+                    chunks.put(c2.id, c2);
+                }
             }
+        } catch (IllegalStateException e) {
+            // the chunk missing where the metadata is stored
+            return false;
         }
         return true;
     }
@@ -2160,45 +2296,45 @@ public class MVStore {
         meta.rollbackTo(version);
         metaChanged = false;
         boolean loadFromFile = false;
-        // get the largest chunk with a version
-        // higher or equal the requested version
-        Chunk removeChunksNewerThan = null;
-        Chunk c = lastChunk;
-        while (true) {
-            if (c == null || c.version < version) {
-                break;
+        // find out which chunks to remove,
+        // and which is the newest chunk to keep
+        // (the chunk list can have gaps)
+        ArrayList<Integer> remove = new ArrayList<Integer>();
+        Chunk keep = null;
+        for (Chunk c : chunks.values()) {
+            if (c.version > version) {
+                remove.add(c.id);
+            } else if (keep == null || keep.id < c.id) {
+                keep = c;
             }
-            removeChunksNewerThan = c;
-            c = chunks.get(c.id - 1);
         }
-        Chunk last = lastChunk;
-        if (removeChunksNewerThan != null &&
-                last.version > removeChunksNewerThan.version) {
+        if (remove.size() > 0) {
+            // remove the youngest first, so we don't create gaps
+            // (in case we remove many chunks)
+            Collections.sort(remove, Collections.reverseOrder());
             revertTemp(version);
             loadFromFile = true;
-            while (true) {
-                last = lastChunk;
-                if (last == null) {
-                    break;
-                } else if (last.version <= removeChunksNewerThan.version) {
-                    break;
-                }
-                chunks.remove(lastChunk.id);
-                long start = last.block * BLOCK_SIZE;
-                int length = last.len * BLOCK_SIZE;
+            for (int id : remove) {
+                Chunk c = chunks.remove(id);
+                long start = c.block * BLOCK_SIZE;
+                int length = c.len * BLOCK_SIZE;
                 fileStore.free(start, length);
-                // need to overwrite the chunk,
-                // so it can not be used
+                // overwrite the chunk,
+                // so it is not be used later on
                 WriteBuffer buff = getWriteBuffer();
                 buff.limit(length);
                 // buff.clear() does not set the data
                 Arrays.fill(buff.getBuffer().array(), (byte) 0);
                 write(start, buff.getBuffer());
                 releaseWriteBuffer(buff);
-                lastChunk = chunks.get(lastChunk.id - 1);
+                // only really needed if we remove many chunks, when writes are
+                // re-ordered - but we do it always, because rollback is not
+                // performance critical
+                sync();
             }
-            writeFileHeader();
-            readFileHeader();
+            lastChunk = keep;
+            writeStoreHeader();
+            readStoreHeader();
         }
         for (MVMap<?, ?> m : New.arrayList(maps.values())) {
             int id = m.getId();
@@ -2210,12 +2346,10 @@ public class MVStore {
                     m.setRootPos(getRootPos(meta, id), -1);
                 }
             }
-
         }
         // rollback might have rolled back the stored chunk metadata as well
         if (lastChunk != null) {
-            c = chunks.get(lastChunk.id - 1);
-            if (c != null) {
+            for (Chunk c : chunks.values()) {
                 meta.put(Chunk.getMetaKey(c.id), c.asString());
             }
         }
@@ -2269,7 +2403,7 @@ public class MVStore {
      * @return the store header
      */
     public Map<String, Object> getStoreHeader() {
-        return fileHeader;
+        return storeHeader;
     }
 
     private void checkOpen() {
@@ -2348,7 +2482,7 @@ public class MVStore {
         // could also commit when there are many unsaved pages,
         // but according to a test it doesn't really help
 
-        long time = getTime();
+        long time = getTimeSinceCreation();
         if (time <= lastCommitTime + autoCommitDelay) {
             return;
         }
@@ -2536,6 +2670,15 @@ public class MVStore {
         return cache;
     }
 
+    /**
+     * Whether the store is read-only.
+     *
+     * @return true if it is
+     */
+    public boolean isReadOnly() {
+        return fileStore == null ? false : fileStore.isReadOnly();
+    }
+
     /**
      * A background writer thread to automatically store changes from time to
      * time.
@@ -2687,6 +2830,17 @@ public class MVStore {
             return set("cacheSize", mb);
         }
 
+        /**
+         * Set the read cache concurrency. The default is 16, meaning 16
+         * segments are used.
+         *
+         * @param concurrency the cache concurrency
+         * @return this
+         */
+        public Builder cacheConcurrency(int concurrency) {
+            return set("cacheConcurrency", concurrency);
+        }
+
         /**
          * Compress data before writing using the LZF algorithm. This will save
          * about 50% of the disk space, but will slow down read and write
