@@ -6,8 +6,9 @@
 package org.h2.schema;
 
 import java.util.ArrayList;
-import java.util.HashMap;
 import java.util.HashSet;
+import java.util.Map;
+import java.util.concurrent.ConcurrentHashMap;
 import org.h2.api.ErrorCode;
 import org.h2.command.ddl.CreateTableData;
 import org.h2.constraint.Constraint;
@@ -37,13 +38,13 @@ public class Schema extends DbObjectBase {
     private User owner;
     private final boolean system;
 
-    private final HashMap<String, Table> tablesAndViews;
-    private final HashMap<String, Index> indexes;
-    private final HashMap<String, Sequence> sequences;
-    private final HashMap<String, TriggerObject> triggers;
-    private final HashMap<String, Constraint> constraints;
-    private final HashMap<String, Constant> constants;
-    private final HashMap<String, FunctionAlias> functions;
+    private final ConcurrentHashMap<String, Table> tablesAndViews;
+    private final ConcurrentHashMap<String, Index> indexes;
+    private final ConcurrentHashMap<String, Sequence> sequences;
+    private final ConcurrentHashMap<String, TriggerObject> triggers;
+    private final ConcurrentHashMap<String, Constraint> constraints;
+    private final ConcurrentHashMap<String, Constant> constants;
+    private final ConcurrentHashMap<String, FunctionAlias> functions;
 
     /**
      * The set of returned unique names that are not yet stored. It is used to
@@ -64,13 +65,13 @@ public class Schema extends DbObjectBase {
      */
     public Schema(Database database, int id, String schemaName, User owner,
             boolean system) {
-        tablesAndViews = database.newStringMap();
-        indexes = database.newStringMap();
-        sequences = database.newStringMap();
-        triggers = database.newStringMap();
-        constraints = database.newStringMap();
-        constants = database.newStringMap();
-        functions = database.newStringMap();
+        tablesAndViews = database.newConcurrentStringMap();
+        indexes = database.newConcurrentStringMap();
+        sequences = database.newConcurrentStringMap();
+        triggers = database.newConcurrentStringMap();
+        constraints = database.newConcurrentStringMap();
+        constants = database.newConcurrentStringMap();
+        functions = database.newConcurrentStringMap();
         initDbObjectBase(database, id, schemaName, Trace.SCHEMA);
         this.owner = owner;
         this.system = system;
@@ -175,8 +176,8 @@ public class Schema extends DbObjectBase {
     }
 
     @SuppressWarnings("unchecked")
-    private HashMap<String, SchemaObject> getMap(int type) {
-        HashMap<String, ? extends SchemaObject> result;
+    private Map<String, SchemaObject> getMap(int type) {
+        Map<String, ? extends SchemaObject> result;
         switch (type) {
         case DbObject.TABLE_OR_VIEW:
             result = tablesAndViews;
@@ -202,7 +203,7 @@ public class Schema extends DbObjectBase {
         default:
             throw DbException.throwInternalError("type=" + type);
         }
-        return (HashMap<String, SchemaObject>) result;
+        return (Map<String, SchemaObject>) result;
     }
 
     /**
@@ -217,7 +218,7 @@ public class Schema extends DbObjectBase {
             DbException.throwInternalError("wrong schema");
         }
         String name = obj.getName();
-        HashMap<String, SchemaObject> map = getMap(obj.getType());
+        Map<String, SchemaObject> map = getMap(obj.getType());
         if (SysProperties.CHECK && map.get(name) != null) {
             DbException.throwInternalError("object already exists: " + name);
         }
@@ -233,7 +234,7 @@ public class Schema extends DbObjectBase {
      */
     public void rename(SchemaObject obj, String newName) {
         int type = obj.getType();
-        HashMap<String, SchemaObject> map = getMap(type);
+        Map<String, SchemaObject> map = getMap(type);
         if (SysProperties.CHECK) {
             if (!map.containsKey(obj.getName())) {
                 DbException.throwInternalError("not found: " + obj.getName());
@@ -357,7 +358,7 @@ public class Schema extends DbObjectBase {
     }
 
     private String getUniqueName(DbObject obj,
-            HashMap<String, ? extends SchemaObject> map, String prefix) {
+            Map<String, ? extends SchemaObject> map, String prefix) {
         String hash = Integer.toHexString(obj.getName().hashCode()).toUpperCase();
         String name = null;
         synchronized (temporaryUniqueNames) {
@@ -390,7 +391,7 @@ public class Schema extends DbObjectBase {
      * @return the unique name
      */
     public String getUniqueConstraintName(Session session, Table table) {
-        HashMap<String, Constraint> tableConstraints;
+        Map<String, Constraint> tableConstraints;
         if (table.isTemporary() && !table.isGlobalTemporary()) {
             tableConstraints = session.getLocalTempTableConstraints();
         } else {
@@ -408,7 +409,7 @@ public class Schema extends DbObjectBase {
      * @return the unique name
      */
     public String getUniqueIndexName(Session session, Table table, String prefix) {
-        HashMap<String, Index> tableIndexes;
+        Map<String, Index> tableIndexes;
         if (table.isTemporary() && !table.isGlobalTemporary()) {
             tableIndexes = session.getLocalTempTableIndexes();
         } else {
@@ -523,7 +524,7 @@ public class Schema extends DbObjectBase {
      * @return a (possible empty) list of all objects
      */
     public ArrayList<SchemaObject> getAll(int type) {
-        HashMap<String, SchemaObject> map = getMap(type);
+        Map<String, SchemaObject> map = getMap(type);
         return New.arrayList(map.values());
     }
 
@@ -557,7 +558,7 @@ public class Schema extends DbObjectBase {
      */
     public void remove(SchemaObject obj) {
         String objName = obj.getName();
-        HashMap<String, SchemaObject> map = getMap(obj.getType());
+        Map<String, SchemaObject> map = getMap(obj.getType());
         if (SysProperties.CHECK && !map.containsKey(objName)) {
             DbException.throwInternalError("not found: " + objName);
         }
