@@ -50,7 +50,6 @@ import org.h2.value.ValueStringIgnoreCase;
 import org.h2.value.ValueTime;
 import org.h2.value.ValueTimestamp;
 import org.h2.value.ValueTimestampTimeZone;
-import org.h2.value.ValueTimestampUtc;
 import org.h2.value.ValueUuid;
 
 /**
@@ -539,12 +538,6 @@ public class Data {
             }
             break;
         }
-        case Value.TIMESTAMP_UTC: {
-            ValueTimestampUtc ts = (ValueTimestampUtc) v;
-            writeByte((byte) type);
-            writeVarLong(ts.getUtcDateTimeNanos());
-            break;
-        }
         case Value.TIMESTAMP_TZ: {
             ValueTimestampTimeZone ts = (ValueTimestampTimeZone) v;
             writeByte((byte) type);
@@ -787,9 +780,6 @@ public class Data {
                     DateTimeUtils.getTimeUTCWithoutDst(readVarLong()),
                     readVarInt());
         }
-        case Value.TIMESTAMP_UTC: {
-            return ValueTimestampUtc.fromNanos(readVarLong());
-        }
         case Value.TIMESTAMP_TZ: {
             long dateValue = readVarLong();
             long nanos = readVarLong();
@@ -1044,10 +1034,6 @@ public class Data {
             return 1 + getVarLongLen(DateTimeUtils.getTimeLocalWithoutDst(ts)) +
                     getVarIntLen(ts.getNanos() % 1000000);
         }
-        case Value.TIMESTAMP_UTC: {
-            ValueTimestampUtc ts = (ValueTimestampUtc) v;
-            return 1 + getVarLongLen(ts.getUtcDateTimeNanos());
-        }
         case Value.TIMESTAMP_TZ: {
             ValueTimestampTimeZone ts = (ValueTimestampTimeZone) v;
             long dateValue = ts.getDateValue();
