@@ -159,14 +159,17 @@ public abstract class Value {
      */
     public static final int GEOMETRY = 22;
     /**
-     * The value type for TIMESTAMP UTC values.
+     * 23 was a short-lived experiment "TIMESTAMP UTC" which has been removed.
      */
-    public static final int TIMESTAMP_UTC = 23;
+    /**
+     * The value type for TIMESTAMP WITH TIMEZONE values.
+     */
+    public static final int TIMESTAMP_TZ = 24;
 
     /**
      * The number of value types.
      */
-    public static final int TYPE_COUNT = TIMESTAMP_UTC + 1;
+    public static final int TYPE_COUNT = TIMESTAMP_TZ;
 
     private static SoftReference<Value[]> softCache =
             new SoftReference<Value[]>(null);
@@ -302,15 +305,15 @@ public abstract class Value {
             return 31;
         case TIMESTAMP:
             return 32;
-        case TIMESTAMP_UTC:
-            return 33;
+        case TIMESTAMP_TZ:
+            return 34;
         case BYTES:
             return 40;
         case BLOB:
             return 41;
-        case UUID:
-            return 42;
         case JAVA_OBJECT:
+            return 42;
+        case UUID:
             return 43;
         case GEOMETRY:
             return 44;
@@ -547,7 +550,7 @@ public abstract class Value {
                 case TIME:
                 case DATE:
                 case TIMESTAMP:
-                case TIMESTAMP_UTC:
+                case TIMESTAMP_TZ:
                 case BYTES:
                 case JAVA_OBJECT:
                 case UUID:
@@ -565,7 +568,6 @@ public abstract class Value {
                 case INT:
                     return ValueByte.get(convertToByte(getInt()));
                 case LONG:
-                case TIMESTAMP_UTC:
                     return ValueByte.get(convertToByte(getLong()));
                 case DECIMAL:
                     return ValueByte.get(convertToByte(convertToLong(getBigDecimal())));
@@ -575,6 +577,9 @@ public abstract class Value {
                     return ValueByte.get(convertToByte(convertToLong(getFloat())));
                 case BYTES:
                     return ValueByte.get((byte) Integer.parseInt(getString(), 16));
+                case TIMESTAMP_TZ:
+                    throw DbException.get(
+                            ErrorCode.DATA_CONVERSION_ERROR_1, getString());
                 }
                 break;
             }
@@ -587,7 +592,6 @@ public abstract class Value {
                 case INT:
                     return ValueShort.get(convertToShort(getInt()));
                 case LONG:
-                case TIMESTAMP_UTC:
                     return ValueShort.get(convertToShort(getLong()));
                 case DECIMAL:
                     return ValueShort.get(convertToShort(convertToLong(getBigDecimal())));
@@ -597,6 +601,9 @@ public abstract class Value {
                     return ValueShort.get(convertToShort(convertToLong(getFloat())));
                 case BYTES:
                     return ValueShort.get((short) Integer.parseInt(getString(), 16));
+                case TIMESTAMP_TZ:
+                    throw DbException.get(
+                            ErrorCode.DATA_CONVERSION_ERROR_1, getString());
                 }
                 break;
             }
@@ -609,7 +616,6 @@ public abstract class Value {
                 case SHORT:
                     return ValueInt.get(getShort());
                 case LONG:
-                case TIMESTAMP_UTC:
                     return ValueInt.get(convertToInt(getLong()));
                 case DECIMAL:
                     return ValueInt.get(convertToInt(convertToLong(getBigDecimal())));
@@ -619,6 +625,9 @@ public abstract class Value {
                     return ValueInt.get(convertToInt(convertToLong(getFloat())));
                 case BYTES:
                     return ValueInt.get((int) Long.parseLong(getString(), 16));
+                case TIMESTAMP_TZ:
+                    throw DbException.get(
+                            ErrorCode.DATA_CONVERSION_ERROR_1, getString());
                 }
                 break;
             }
@@ -646,8 +655,9 @@ public abstract class Value {
                     }
                     return ValueLong.get(Long.parseLong(getString(), 16));
                 }
-                case TIMESTAMP_UTC:
-                    return ValueLong.get(getLong());
+                case TIMESTAMP_TZ:
+                    throw DbException.get(
+                            ErrorCode.DATA_CONVERSION_ERROR_1, getString());
                 }
                 break;
             }
@@ -663,7 +673,6 @@ public abstract class Value {
                 case INT:
                     return ValueDecimal.get(BigDecimal.valueOf(getInt()));
                 case LONG:
-                case TIMESTAMP_UTC:
                     return ValueDecimal.get(BigDecimal.valueOf(getLong()));
                 case DOUBLE: {
                     double d = getDouble();
@@ -682,6 +691,9 @@ public abstract class Value {
                     // better rounding behavior than BigDecimal.valueOf(f)
                     return ValueDecimal.get(new BigDecimal(Float.toString(f)));
                 }
+                case TIMESTAMP_TZ:
+                    throw DbException.get(
+                            ErrorCode.DATA_CONVERSION_ERROR_1, getString());
                 }
                 break;
             }
@@ -696,12 +708,14 @@ public abstract class Value {
                 case INT:
                     return ValueDouble.get(getInt());
                 case LONG:
-                case TIMESTAMP_UTC:
                     return ValueDouble.get(getLong());
                 case DECIMAL:
                     return ValueDouble.get(getBigDecimal().doubleValue());
                 case FLOAT:
                     return ValueDouble.get(getFloat());
+                case TIMESTAMP_TZ:
+                    throw DbException.get(
+                            ErrorCode.DATA_CONVERSION_ERROR_1, getString());
                 }
                 break;
             }
@@ -716,12 +730,14 @@ public abstract class Value {
                 case INT:
                     return ValueFloat.get(getInt());
                 case LONG:
-                case TIMESTAMP_UTC:
                     return ValueFloat.get(getLong());
                 case DECIMAL:
                     return ValueFloat.get(getBigDecimal().floatValue());
                 case DOUBLE:
                     return ValueFloat.get((float) getDouble());
+                case TIMESTAMP_TZ:
+                    throw DbException.get(
+                            ErrorCode.DATA_CONVERSION_ERROR_1, getString());
                 }
                 break;
             }
@@ -735,9 +751,9 @@ public abstract class Value {
                 case TIMESTAMP:
                     return ValueDate.fromDateValue(
                             ((ValueTimestamp) this).getDateValue());
-                case TIMESTAMP_UTC:
-                    return ValueDate.fromMillis(
-                            ((ValueTimestampUtc) this).getUtcDateTimeMillis());
+                case TIMESTAMP_TZ:
+                    return ValueDate.fromDateValue(
+                            ((ValueTimestampTimeZone) this).getDateValue());
                 }
                 break;
             }
@@ -750,9 +766,9 @@ public abstract class Value {
                 case TIMESTAMP:
                     return ValueTime.fromNanos(
                             ((ValueTimestamp) this).getTimeNanos());
-                case TIMESTAMP_UTC:
-                    return ValueTime.fromMillis(
-                            ((ValueTimestampUtc) this).getUtcDateTimeMillis());
+                case TIMESTAMP_TZ:
+                    return ValueTime.fromNanos(
+                            ((ValueTimestampTimeZone) this).getTimeNanos());
                 }
                 break;
             }
@@ -764,35 +780,10 @@ public abstract class Value {
                 case DATE:
                     return ValueTimestamp.fromDateValueAndNanos(
                             ((ValueDate) this).getDateValue(), 0);
-                case TIMESTAMP_UTC:
-                    return ValueTimestamp.fromMillisNanos(
-                            ((ValueTimestampUtc) this).getUtcDateTimeMillis(),
-                            ((ValueTimestampUtc) this).getNanosSinceLastMillis());
-                }
-                break;
-            }
-            case TIMESTAMP_UTC: {
-                switch (getType()) {
-                case BOOLEAN:
-                    return ValueTimestampUtc.fromNanos(getBoolean().booleanValue() ? 1 : 0);
-                case BYTE:
-                    return ValueTimestampUtc.fromNanos(getByte());
-                case SHORT:
-                    return ValueTimestampUtc.fromNanos(getShort());
-                case INT:
-                    return ValueTimestampUtc.fromNanos(getInt());
-                case LONG:
-                    return ValueTimestampUtc.fromNanos(getLong());
-                case DECIMAL:
-                    return ValueTimestampUtc.fromNanos(getBigDecimal().longValue());
-                case FLOAT:
-                    return ValueTimestampUtc.fromNanos((long) getFloat());
-                case DOUBLE:
-                    return ValueTimestampUtc.fromNanos((long) getDouble());
-                case TIMESTAMP:
-                    return ValueTimestampUtc.fromMillisNanos(
-                            ((ValueTimestamp) this).getTimestamp().getTime(),
-                            ((ValueTimestamp) this).getTimestamp().getNanos());
+                case TIMESTAMP_TZ:
+                    return ValueTimestamp.fromDateValueAndNanos(
+                            ((ValueTimestampTimeZone) this).getDateValue(),
+                            ((ValueTimestampTimeZone) this).getTimeNanos());
                 }
                 break;
             }
@@ -835,6 +826,9 @@ public abstract class Value {
                             (byte) x
                     });
                 }
+                case TIMESTAMP_TZ:
+                    throw DbException.get(
+                            ErrorCode.DATA_CONVERSION_ERROR_1, getString());
                 }
                 break;
             }
@@ -844,6 +838,9 @@ public abstract class Value {
                 case BLOB:
                     return ValueJavaObject.getNoCopy(
                             null, getBytesNoCopy(), getDataHandler());
+                case TIMESTAMP_TZ:
+                    throw DbException.get(
+                            ErrorCode.DATA_CONVERSION_ERROR_1, getString());
                 }
                 break;
             }
@@ -852,6 +849,9 @@ public abstract class Value {
                 case BYTES:
                     return ValueLobDb.createSmallLob(
                             Value.BLOB, getBytesNoCopy());
+                case TIMESTAMP_TZ:
+                    throw DbException.get(
+                            ErrorCode.DATA_CONVERSION_ERROR_1, getString());
                 }
                 break;
             }
@@ -859,9 +859,23 @@ public abstract class Value {
                 switch (getType()) {
                 case BYTES:
                     return ValueUuid.get(getBytesNoCopy());
+                case JAVA_OBJECT:
+                    Object object = JdbcUtils.deserialize(getBytesNoCopy(),
+                            getDataHandler());
+                    if (object instanceof java.util.UUID) {
+                        java.util.UUID uuid = (java.util.UUID) object;
+                        return ValueUuid.get(uuid.getMostSignificantBits(),
+                                uuid.getLeastSignificantBits());
+                    } else {
+                        throw DbException.get(ErrorCode.DATA_CONVERSION_ERROR_1, getString());
+                    }
+                case TIMESTAMP_TZ:
+                    throw DbException.get(
+                            ErrorCode.DATA_CONVERSION_ERROR_1, getString());
                 }
+                break;
             }
-            case GEOMETRY:
+            case GEOMETRY: {
                 switch (getType()) {
                 case BYTES:
                     return ValueGeometry.get(getBytesNoCopy());
@@ -870,7 +884,12 @@ public abstract class Value {
                     if (DataType.isGeometry(object)) {
                         return ValueGeometry.getFromGeometry(object);
                     }
+                case TIMESTAMP_TZ:
+                    throw DbException.get(
+                            ErrorCode.DATA_CONVERSION_ERROR_1, getString());
                 }
+                break;
+            }
             }
             // conversion by parsing the string value
             String s = getString();
@@ -909,8 +928,8 @@ public abstract class Value {
                 return ValueDate.parse(s.trim());
             case TIMESTAMP:
                 return ValueTimestamp.parse(s.trim());
-            case TIMESTAMP_UTC:
-                return ValueTimestampUtc.parse(s.trim());
+            case TIMESTAMP_TZ:
+                return ValueTimestampTimeZone.parse(s.trim());
             case BYTES:
                 return ValueBytes.getNoCopy(
                         StringUtils.convertHexToBytes(s.trim()));
