@@ -53,6 +53,7 @@ import org.h2.test.db.TestPowerOff;
 import org.h2.test.db.TestQueryCache;
 import org.h2.test.db.TestReadOnly;
 import org.h2.test.db.TestRecursiveQueries;
+import org.h2.test.db.TestReplace;
 import org.h2.test.db.TestRights;
 import org.h2.test.db.TestRowFactory;
 import org.h2.test.db.TestRunscript;
@@ -206,7 +207,6 @@ import org.h2.test.unit.TestSort;
 import org.h2.test.unit.TestStreams;
 import org.h2.test.unit.TestStringCache;
 import org.h2.test.unit.TestStringUtils;
-import org.h2.test.unit.TestTimeStampUtc;
 import org.h2.test.unit.TestTimeStampWithTimeZone;
 import org.h2.test.unit.TestTools;
 import org.h2.test.unit.TestTraceSystem;
@@ -266,7 +266,7 @@ java org.h2.test.TestAll timer
     /**
      * Whether the MVStore storage is used.
      */
-    public final boolean mvStore = Constants.VERSION_MINOR >= 4;
+    public boolean mvStore = Constants.VERSION_MINOR >= 4;
 
     /**
      * If the test should run with many rows.
@@ -298,6 +298,11 @@ java org.h2.test.TestAll timer
      */
     public boolean mvcc = mvStore;
 
+    /**
+     * If the multi-threaded mode should be used.
+     */
+    public boolean multiThreaded;
+
     /**
      * The cipher to use (null for unencrypted).
      */
@@ -712,6 +717,7 @@ kill -9 `jps -l | grep "org.h2.test." | cut -d " " -f 1`
         addTest(new TestView());
         addTest(new TestViewAlterTable());
         addTest(new TestViewDropView());
+        addTest(new TestReplace());
 
         // jaqu
         addTest(new AliasMapTest());
@@ -858,7 +864,6 @@ kill -9 `jps -l | grep "org.h2.test." | cut -d " " -f 1`
         addTest(new TestSort());
         addTest(new TestStreams());
         addTest(new TestStringUtils());
-        addTest(new TestTimeStampUtc());
         addTest(new TestTimeStampWithTimeZone());
         addTest(new TestTraceSystem());
         addTest(new TestUpgrade());
