@@ -37,6 +37,7 @@ public class TestCompatibilityOracle extends TestBase {
         testDecimalScale();
         testPoundSymbolInColumnName();
         testToDate();
+        testForbidEmptyInClause();
     }
 
     private void testTreatEmptyStringsAsNull() throws SQLException {
@@ -161,6 +162,21 @@ public class TestCompatibilityOracle extends TestBase {
         conn.close();
     }
 
+    private void testForbidEmptyInClause() throws SQLException {
+        deleteDb("oracle");
+        Connection conn = getConnection("oracle;MODE=Oracle");
+        Statement stat = conn.createStatement();
+
+        stat.execute("CREATE TABLE A (ID NUMBER, X VARCHAR2(1))");
+        try {
+            stat.executeQuery("SELECT * FROM A WHERE ID IN ()");
+            fail();
+        } catch (SQLException e) {
+        } finally {
+            conn.close();
+        }
+    }
+
     private void assertResultDate(String expected, Statement stat, String sql)
             throws SQLException {
         SimpleDateFormat iso8601 = new SimpleDateFormat(
