@@ -14,7 +14,6 @@ import java.sql.ResultSetMetaData;
 import java.sql.SQLException;
 import java.sql.Statement;
 import java.sql.Timestamp;
-
 import org.h2.api.ErrorCode;
 import org.h2.store.fs.FileUtils;
 import org.h2.test.TestBase;
@@ -714,12 +713,9 @@ public class TestLinkedTable extends TestBase {
         String sql = "CREATE LINKED TABLE T(NULL, " +
                 "'jdbc:h2:mem:one', 'sa', 'sa', 'TEST') READONLY";
         sb.execute(sql);
-        ResultSet rs = sb.executeQuery("SELECT * FROM T");
-        try {
+        try (ResultSet rs = sb.executeQuery("SELECT * FROM T")) {
             assertTrue(rs.next());
             assertEquals("POINT (1 1)", rs.getString("THE_GEOM"));
-        } finally {
-            rs.close();
         }
         sb.execute("DROP TABLE T");
         ca.close();
