@@ -12,23 +12,22 @@ import java.sql.Savepoint;
 import java.sql.Statement;
 import java.sql.Types;
 import java.util.Random;
-
-import com.vividsolutions.jts.geom.Envelope;
-import com.vividsolutions.jts.geom.Point;
-import com.vividsolutions.jts.geom.util.AffineTransformation;
 import org.h2.api.Aggregate;
 import org.h2.test.TestBase;
 import org.h2.tools.SimpleResultSet;
 import org.h2.tools.SimpleRowSource;
 import org.h2.value.DataType;
 import org.h2.value.Value;
+import org.h2.value.ValueGeometry;
 import com.vividsolutions.jts.geom.Coordinate;
+import com.vividsolutions.jts.geom.Envelope;
 import com.vividsolutions.jts.geom.Geometry;
 import com.vividsolutions.jts.geom.GeometryFactory;
+import com.vividsolutions.jts.geom.Point;
 import com.vividsolutions.jts.geom.Polygon;
+import com.vividsolutions.jts.geom.util.AffineTransformation;
 import com.vividsolutions.jts.io.ParseException;
 import com.vividsolutions.jts.io.WKTReader;
-import org.h2.value.ValueGeometry;
 
 /**
  * Spatial datatype and index tests.
@@ -93,6 +92,8 @@ public class TestSpatial extends TestBase {
         testNullableGeometryDelete();
         testNullableGeometryInsert();
         testNullableGeometryUpdate();
+        testIndexUpdateNullGeometry();
+        testInsertNull();
     }
 
     private void testBug1() throws SQLException {
@@ -188,8 +189,7 @@ public class TestSpatial extends TestBase {
 
     private void testOverlap() throws SQLException {
         deleteDb("spatial");
-        Connection conn = getConnection(URL);
-        try {
+        try (Connection conn = getConnection(URL)) {
             Statement stat = conn.createStatement();
             stat.execute("create memory table test" +
                     "(id int primary key, poly geometry)");
@@ -207,14 +207,11 @@ public class TestSpatial extends TestBase {
             assertEquals(1, rs.getInt("id"));
             assertFalse(rs.next());
             stat.execute("drop table test");
-        } finally {
-            conn.close();
         }
     }
     private void testPersistentSpatialIndex() throws SQLException {
         deleteDb("spatial");
-        Connection conn = getConnection(URL);
-        try {
+        try (Connection conn = getConnection(URL)) {
             Statement stat = conn.createStatement();
             stat.execute("create table test" +
                     "(id int primary key, poly geometry)");
@@ -245,17 +242,13 @@ public class TestSpatial extends TestBase {
             assertEquals(1, rs.getInt("id"));
             assertFalse(rs.next());
             rs.close();
-        } finally {
-            // Close the database
-            conn.close();
         }
 
         if (config.memory) {
             return;
         }
 
-        conn = getConnection(URL);
-        try {
+        try (Connection conn = getConnection(URL)) {
             Statement stat = conn.createStatement();
             ResultSet rs = stat.executeQuery(
                     "select * from test " +
@@ -264,15 +257,12 @@ public class TestSpatial extends TestBase {
             assertEquals(1, rs.getInt("id"));
             assertFalse(rs.next());
             stat.execute("drop table test");
-        } finally {
-            conn.close();
         }
-
     }
+
     private void testNotOverlap() throws SQLException {
         deleteDb("spatial");
-        Connection conn = getConnection(URL);
-        try {
+        try (Connection conn = getConnection(URL)) {
             Statement stat = conn.createStatement();
             stat.execute("create memory table test" +
                     "(id int primary key, poly geometry)");
@@ -294,8 +284,6 @@ public class TestSpatial extends TestBase {
             assertEquals(5, rs.getInt("id"));
             assertFalse(rs.next());
             stat.execute("drop table test");
-        } finally {
-            conn.close();
         }
     }
 
@@ -348,14 +336,10 @@ public class TestSpatial extends TestBase {
 
     private void testSpatialIndexQueryMultipleTable() throws SQLException {
         deleteDb("spatial");
-        Connection conn = getConnection(URL);
-        try {
+        try (Connection conn = getConnection(URL)) {
             Statement stat = conn.createStatement();
             createTestTable(stat);
             testRoadAndArea(stat);
-        } finally {
-            // Close the database
-            conn.close();
         }
         deleteDb("spatial");
     }
@@ -389,9 +373,8 @@ public class TestSpatial extends TestBase {
     private void testIndexTransaction() throws SQLException {
         // Check session management in index
         deleteDb("spatial");
-        Connection conn = getConnection(URL);
-        conn.setAutoCommit(false);
-        try {
+        try (Connection conn = getConnection(URL)) {
+            conn.setAutoCommit(false);
             Statement stat = conn.createStatement();
             createTestTable(stat);
             Savepoint sp = conn.setSavepoint();
@@ -427,10 +410,7 @@ public class TestSpatial extends TestBase {
             conn.rollback(sp);
             // Check if the index is restored
             testRoadAndArea(stat);
-        } finally {
-            conn.close();
         }
-
     }
 
     /**
@@ -501,8 +481,7 @@ public class TestSpatial extends TestBase {
      */
     private void testJavaAlias() throws SQLException {
         deleteDb("spatial");
-        Connection conn = getConnection(URL);
-        try {
+        try (Connection conn = getConnection(URL)) {
             Statement stat = conn.createStatement();
             stat.execute("CREATE ALIAS T_GEOM_FROM_TEXT FOR \"" +
                     TestSpatial.class.getName() + ".geomFromText\"");
@@ -516,8 +495,6 @@ public class TestSpatial extends TestBase {
             assertTrue(rs.next());
             assertEquals("POLYGON ((62 48, 84 48, 84 42, 56 34, 62 48))",
                     rs.getObject(1).toString());
-        } finally {
-            conn.close();
         }
         deleteDb("spatial");
     }
@@ -527,8 +504,7 @@ public class TestSpatial extends TestBase {
      */
     private void testJavaAliasTableFunction() throws SQLException {
         deleteDb("spatial");
-        Connection conn = getConnection(URL);
-        try {
+        try (Connection conn = getConnection(URL)) {
             Statement stat = conn.createStatement();
             stat.execute("CREATE ALIAS T_RANDOM_GEOM_TABLE FOR \"" +
                     TestSpatial.class.getName() + ".getRandomGeometryTable\"");
@@ -539,8 +515,6 @@ public class TestSpatial extends TestBase {
             ResultSet rs = stat.executeQuery("select count(*) from test");
             assertTrue(rs.next());
             assertEquals(20, rs.getInt(1));
-        } finally {
-            conn.close();
         }
         deleteDb("spatial");
     }
@@ -694,8 +668,7 @@ public class TestSpatial extends TestBase {
      */
     private void testTableFunctionGeometry() throws SQLException {
         deleteDb("spatial");
-        Connection conn = getConnection(URL);
-        try {
+        try (Connection conn = getConnection(URL)) {
             Statement stat = conn.createStatement();
             stat.execute("CREATE ALIAS POINT_TABLE FOR \"" +
                     TestSpatial.class.getName() + ".pointTable\"");
@@ -707,8 +680,6 @@ public class TestSpatial extends TestBase {
             assertEquals("geometry",
                     columnMeta.getString("TYPE_NAME").toLowerCase());
             assertFalse(columnMeta.next());
-        } finally {
-            conn.close();
         }
         deleteDb("spatial");
     }
@@ -730,8 +701,7 @@ public class TestSpatial extends TestBase {
 
     private void testAggregateWithGeometry() throws SQLException {
         deleteDb("spatialIndex");
-        Connection conn = getConnection("spatialIndex");
-        try {
+        try (Connection conn = getConnection("spatialIndex")) {
             Statement st = conn.createStatement();
             st.execute("CREATE AGGREGATE TABLE_ENVELOPE FOR \""+
                     TableEnvelope.class.getName()+"\"");
@@ -745,8 +715,6 @@ public class TestSpatial extends TestBase {
             assertTrue(new Envelope(1, 10, 1, 5).equals(
                     ((Geometry) rs.getObject(1)).getEnvelopeInternal()));
             assertFalse(rs.next());
-        } finally {
-            conn.close();
         }
         deleteDb("spatialIndex");
     }
@@ -791,8 +759,7 @@ public class TestSpatial extends TestBase {
 
     private void testTableViewSpatialPredicate() throws SQLException {
         deleteDb("spatial");
-        Connection conn = getConnection(URL);
-        try {
+        try (Connection conn = getConnection(URL)) {
             Statement stat = conn.createStatement();
             stat.execute("drop table if exists test");
             stat.execute("drop view if exists test_view");
@@ -817,9 +784,6 @@ public class TestSpatial extends TestBase {
             assertEquals(1, rs.getInt("id"));
             assertFalse(rs.next());
             rs.close();
-        } finally {
-            // Close the database
-            conn.close();
         }
         deleteDb("spatial");
     }
@@ -829,16 +793,13 @@ public class TestSpatial extends TestBase {
      */
     private void testValueGeometryScript() throws SQLException {
         ValueGeometry valueGeometry = ValueGeometry.get("POINT(1 1 5)");
-        Connection conn = getConnection(URL);
-        try {
+        try (Connection conn = getConnection(URL)) {
             ResultSet rs = conn.createStatement().executeQuery(
                     "SELECT " + valueGeometry.getSQL());
             assertTrue(rs.next());
             Object obj = rs.getObject(1);
             ValueGeometry g = ValueGeometry.getFromGeometry(obj);
             assertTrue("got: " + g + " exp: " + valueGeometry, valueGeometry.equals(g));
-        } finally {
-            conn.close();
         }
     }
 
@@ -847,8 +808,7 @@ public class TestSpatial extends TestBase {
      * be updated.
      */
     private void testInPlaceUpdate() throws SQLException {
-        Connection conn = getConnection(URL);
-        try {
+        try (Connection conn = getConnection(URL)) {
             ResultSet rs = conn.createStatement().executeQuery(
                     "SELECT 'POINT(1 1)'::geometry");
             assertTrue(rs.next());
@@ -863,15 +823,12 @@ public class TestSpatial extends TestBase {
             assertEquals(1, ((Point) rs.getObject(1)).getX());
             assertEquals(1, ((Point) rs.getObject(1)).getY());
             rs.close();
-        } finally {
-            conn.close();
         }
     }
 
     private void testScanIndexOnNonSpatialQuery() throws SQLException {
         deleteDb("spatial");
-        Connection conn = getConnection(URL);
-        try {
+        try (Connection conn = getConnection(URL)) {
             Statement stat = conn.createStatement();
             stat.execute("drop table if exists test");
             stat.execute("create table test(id serial primary key, " +
@@ -879,18 +836,15 @@ public class TestSpatial extends TestBase {
             stat.execute("create spatial index spatial on test(the_geom)");
             ResultSet rs = stat.executeQuery("explain select * from test where _ROWID_ = 5");
             assertTrue(rs.next());
-            assertContains(rs.getString(1), "/* PUBLIC.SPATIAL: _ROWID_ = 5 */");
-        } finally {
-            // Close the database
-            conn.close();
+            assertFalse(rs.getString(1).contains("/* PUBLIC.SPATIAL: _ROWID_ = " +
+                    "5 */"));
         }
         deleteDb("spatial");
     }
 
     private void testStoreCorruption() throws SQLException {
         deleteDb("spatial");
-        Connection conn = getConnection(URL);
-        try {
+        try (Connection conn = getConnection(URL)) {
             Statement stat = conn.createStatement();
             stat.execute("drop table if exists pt_cloud;\n" +
                     "CREATE TABLE PT_CLOUD AS " +
@@ -909,17 +863,13 @@ public class TestSpatial extends TestBase {
                     " system_range(1e6,1e6+50) A,system_range(6e6,6e6+50) B;\n" +
                     "create spatial index pt_index on pt_cloud(the_geom);\n" +
                     "shutdown compact;");
-        } finally {
-            // Close the database
-            conn.close();
         }
         deleteDb("spatial");
     }
 
     private void testExplainSpatialIndexWithPk() throws SQLException {
         deleteDb("spatial");
-        Connection conn = getConnection(URL);
-        try {
+        try (Connection conn = getConnection(URL)) {
             Statement stat = conn.createStatement();
             stat.execute("drop table if exists pt_cloud;");
             stat.execute("CREATE TABLE PT_CLOUD(id serial, the_geom geometry) AS " +
@@ -936,9 +886,6 @@ public class TestSpatial extends TestBase {
             } finally {
                 rs.close();
             }
-        } finally {
-            // Close the database
-            conn.close();
         }
         deleteDb("spatial");
     }
@@ -1076,4 +1023,119 @@ public class TestSpatial extends TestBase {
         deleteDb("spatial");
     }
 
+    private void testIndexUpdateNullGeometry() throws SQLException {
+        deleteDb("spatial");
+        Connection conn = getConnection(URL);
+        Statement stat = conn.createStatement();
+        stat.execute("drop table if exists DUMMY_11;");
+        stat.execute("CREATE TABLE PUBLIC.DUMMY_11 (fid serial,  GEOM GEOMETRY);");
+        stat.execute("CREATE SPATIAL INDEX PUBLIC_DUMMY_11_SPATIAL_INDEX on" +
+                " PUBLIC.DUMMY_11(GEOM);");
+        stat.execute("insert into PUBLIC.DUMMY_11(geom) values(null);");
+        stat.execute("update PUBLIC.DUMMY_11 set geom =" +
+                " 'POLYGON ((1 1, 5 1, 5 5, 1 5, 1 1))';");
+        ResultSet rs = stat.executeQuery("select fid, GEOM from DUMMY_11 " +
+                "where  GEOM && " +
+                "'POLYGON" +
+                "((1 1,5 1,5 5,1 5,1 1))';");
+        try {
+            assertTrue(rs.next());
+            assertEquals("POLYGON ((1 1, 5 1, 5 5, 1 5, 1 1))", rs.getString(2));
+        } finally {
+            rs.close();
+        }
+        // Update again the geometry elsewhere
+        stat.execute("update PUBLIC.DUMMY_11 set geom =" +
+                " 'POLYGON ((10 10, 50 10, 50 50, 10 50, 10 10))';");
+
+        rs = stat.executeQuery("select fid, GEOM from DUMMY_11 " +
+                "where  GEOM && " +
+                "'POLYGON ((10 10, 50 10, 50 50, 10 50, 10 10))';");
+        try {
+            assertTrue(rs.next());
+            assertEquals("POLYGON ((10 10, 50 10, 50 50, 10 50, 10 10))", rs.getString(2));
+        } finally {
+            rs.close();
+        }
+        conn.close();
+        deleteDb("spatial");
+    }
+
+    private void testInsertNull() throws SQLException {
+        deleteDb("spatial");
+        Connection conn = getConnection(URL);
+        Statement stat = conn.createStatement();
+        stat.execute("\n" +
+            "drop table if exists PUBLIC.DUMMY_12;\n" +
+            "CREATE TABLE PUBLIC.DUMMY_12 (\n" +
+            "    \"fid\" serial,\n" +
+            "    Z_ID INTEGER,\n" +
+            "    GEOM GEOMETRY,\n" +
+            "    CONSTRAINT CONSTRAINT_DUMMY_12 PRIMARY KEY (\"fid\")\n" +
+            ");\n" +
+            "CREATE INDEX PRIMARY_KEY_DUMMY_12 ON PUBLIC.DUMMY_12 (\"fid\");\n" +
+            "CREATE spatial INDEX PUBLIC_DUMMY_12_SPATIAL_INDEX_ ON PUBLIC.DUMMY_12 (GEOM);\n" +
+            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (123,3125163,NULL);\n" +
+            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (124,3125164,NULL);\n" +
+            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (125,3125173,NULL);\n" +
+            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (126,3125174,NULL);\n" +
+            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (127,3125175,NULL);\n" +
+            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (128,3125176,NULL);\n" +
+            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (129,3125177,NULL);\n" +
+            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (130,3125178,NULL);\n" +
+            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (131,3125179,NULL);\n" +
+            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (132,3125180,NULL);\n" +
+            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (133,3125335,NULL);\n" +
+            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (134,3125336,NULL);\n" +
+            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (135,3125165,NULL);\n" +
+            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (136,3125337,NULL);\n" +
+            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (137,3125338,NULL);\n" +
+            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (138,3125339,NULL);\n" +
+            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (139,3125340,NULL);\n" +
+            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (140,3125341,NULL);\n" +
+            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (141,3125342,NULL);\n" +
+            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (142,3125343,NULL);\n" +
+            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (143,3125344,NULL);\n" +
+            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (144,3125345,NULL);\n" +
+            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (145,3125346,NULL);\n" +
+            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (146,3125166,NULL);\n" +
+            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (147,3125347,NULL);\n" +
+            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (148,3125348,NULL);\n" +
+            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (149,3125349,NULL);\n" +
+            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (150,3125350,NULL);\n" +
+            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (151,3125351,NULL);\n" +
+            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (152,3125352,NULL);\n" +
+            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (153,3125353,NULL);\n" +
+            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (154,3125354,NULL);\n" +
+            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (155,3125355,NULL);\n" +
+            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (156,3125356,NULL);\n" +
+            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (157,3125167,NULL);\n" +
+            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (158,3125357,NULL);\n" +
+            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (159,3125358,NULL);\n" +
+            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (160,3125359,NULL);\n" +
+            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (161,3125360,NULL);\n" +
+            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (162,3125361,NULL);\n" +
+            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (163,3125362,NULL);\n" +
+            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (164,3125363,NULL);\n" +
+            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (165,3125364,NULL);\n" +
+            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (166,3125365,NULL);\n" +
+            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (167,3125366,NULL);\n" +
+            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (168,3125168,NULL);\n" +
+            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (169,3125367,NULL);\n" +
+            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (170,3125368,NULL);\n" +
+            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (171,3125369,NULL);\n" +
+            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (172,3125370,NULL);\n" +
+            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (173,3125169,NULL);\n" +
+            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (174,3125170,NULL);\n" +
+            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (175,3125171,NULL);\n" +
+            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (176,3125172,NULL);\n" +
+            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (177,-2,NULL);\n" +
+            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (178,-1,NULL);\n" +
+            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (179," +
+            "-1,NULL);");
+        try (ResultSet rs = stat.executeQuery("select * from DUMMY_12")) {
+            assertTrue(rs.next());
+        }
+
+    }
 }
