@@ -1348,7 +1348,8 @@ public class TestTableEngines extends TestBase {
         }
 
         @Override
-        public IndexLookupBatch createLookupBatch(final TableFilter filter) {
+        public IndexLookupBatch createLookupBatch(TableFilter[] filters, int f) {
+            final TableFilter filter = filters[f];
             assert0(filter.getMasks() != null || "scan".equals(getName()), "masks");
             final int preferredSize = preferredBatchSize;
             if (preferredSize == 0) {
