@@ -27,6 +27,7 @@ import org.h2.test.TestBase;
 import org.h2.tools.SimpleResultSet;
 import org.h2.util.IOUtils;
 import org.h2.util.JdbcUtils;
+import org.h2.util.LocalDateTimeUtils;
 import org.h2.util.Utils;
 
 /**
@@ -147,6 +148,7 @@ public class TestCallableStatement extends TestBase {
         assertEquals(1, call.getLong(1));
         assertEquals(1, call.getByte(1));
         assertEquals(1, ((Long) call.getObject(1)).longValue());
+        assertEquals(1, call.getObject(1, Long.class).longValue());
         assertFalse(call.wasNull());
 
         call.setFloat(2, 1.1f);
@@ -169,17 +171,29 @@ public class TestCallableStatement extends TestBase {
         call.registerOutParameter(1, Types.DATE);
         call.execute();
         assertEquals("2000-01-01", call.getDate(1).toString());
+        if (LocalDateTimeUtils.isJava8DateApiPresent()) {
+            assertEquals("2000-01-01", call.getObject(1,
+                            LocalDateTimeUtils.getLocalDateClass()).toString());
+        }
 
         call.setTime(2, java.sql.Time.valueOf("01:02:03"));
         call.registerOutParameter(1, Types.TIME);
         call.execute();
         assertEquals("01:02:03", call.getTime(1).toString());
+        if (LocalDateTimeUtils.isJava8DateApiPresent()) {
+            assertEquals("01:02:03", call.getObject(1,
+                            LocalDateTimeUtils.getLocalTimeClass()).toString());
+        }
 
         call.setTimestamp(2, java.sql.Timestamp.valueOf(
                 "2001-02-03 04:05:06.789"));
         call.registerOutParameter(1, Types.TIMESTAMP);
         call.execute();
         assertEquals("2001-02-03 04:05:06.789", call.getTimestamp(1).toString());
+        if (LocalDateTimeUtils.isJava8DateApiPresent()) {
+            assertEquals("2001-02-03T04:05:06.789", call.getObject(1,
+                            LocalDateTimeUtils.getLocalDateTimeClass()).toString());
+        }
 
         call.setBoolean(2, true);
         call.registerOutParameter(1, Types.BIT);
@@ -263,10 +277,28 @@ public class TestCallableStatement extends TestBase {
 
         assertEquals("2001-02-03 10:20:30.0", call.getTimestamp(4).toString());
         assertEquals("2001-02-03 10:20:30.0", call.getTimestamp("D").toString());
+        if (LocalDateTimeUtils.isJava8DateApiPresent()) {
+            assertEquals("2001-02-03T10:20:30", call.getObject(4,
+                            LocalDateTimeUtils.getLocalDateTimeClass()).toString());
+            assertEquals("2001-02-03T10:20:30", call.getObject("D",
+                            LocalDateTimeUtils.getLocalDateTimeClass()).toString());
+        }
         assertEquals("10:20:30", call.getTime(4).toString());
         assertEquals("10:20:30", call.getTime("D").toString());
+        if (LocalDateTimeUtils.isJava8DateApiPresent()) {
+            assertEquals("10:20:30", call.getObject(4,
+                            LocalDateTimeUtils.getLocalTimeClass()).toString());
+            assertEquals("10:20:30", call.getObject("D",
+                            LocalDateTimeUtils.getLocalTimeClass()).toString());
+        }
         assertEquals("2001-02-03", call.getDate(4).toString());
         assertEquals("2001-02-03", call.getDate("D").toString());
+        if (LocalDateTimeUtils.isJava8DateApiPresent()) {
+            assertEquals("2001-02-03", call.getObject(4,
+                            LocalDateTimeUtils.getLocalDateClass()).toString());
+            assertEquals("2001-02-03", call.getObject("D",
+                            LocalDateTimeUtils.getLocalDateClass()).toString());
+        }
 
         assertEquals(100, call.getInt(1));
         assertEquals(100, call.getInt("A"));
