@@ -8,6 +8,7 @@ package org.h2.test.store;
 import java.io.ByteArrayInputStream;
 import java.io.IOException;
 import java.io.InputStream;
+import java.io.StringReader;
 import java.math.BigDecimal;
 import java.nio.channels.FileChannel;
 import java.sql.Connection;
@@ -17,6 +18,7 @@ import java.sql.ResultSet;
 import java.sql.SQLException;
 import java.sql.Savepoint;
 import java.sql.Statement;
+import java.util.concurrent.atomic.AtomicBoolean;
 
 import org.h2.api.ErrorCode;
 import org.h2.engine.Constants;
@@ -27,7 +29,6 @@ import org.h2.mvstore.MVStore;
 import org.h2.mvstore.db.TransactionStore;
 import org.h2.store.fs.FileUtils;
 import org.h2.test.TestBase;
-import org.h2.tools.DeleteDbFiles;
 import org.h2.tools.Recover;
 import org.h2.tools.Restore;
 import org.h2.util.JdbcUtils;
@@ -49,6 +50,11 @@ public class TestMVTableEngine extends TestBase {
 
     @Override
     public void test() throws Exception {
+        testLobCopy();
+        testLobReuse();
+        testShutdownDuringLobCreation();
+        testLobCreationThenShutdown();
+        testManyTransactions();
         testAppendOnly();
         testLowRetentionTime();
         testOldAndNew();
@@ -82,10 +88,179 @@ public class TestMVTableEngine extends TestBase {
         testSimple();
     }
 
+    private void testLobCopy() throws Exception {
+        deleteDb(getTestName());
+        Connection conn = getConnection(getTestName());
+        Statement stat = conn.createStatement();
+        stat.execute("create table test(id int primary key, data clob)");
+        stat = conn.createStatement();
+        stat.execute("insert into test(id, data) values(2, space(300))");
+        stat.execute("insert into test(id, data) values(1, space(300))");
+        stat.execute("alter table test add column x int");
+        if (!config.memory) {
+            conn.close();
+            conn = getConnection(getTestName());
+        }
+        stat = conn.createStatement();
+        ResultSet rs = stat.executeQuery("select data from test");
+        while (rs.next()) {
+            rs.getString(1);
+        }
+        conn.close();
+    }
+
+    private void testLobReuse() throws Exception {
+        deleteDb(getTestName());
+        Connection conn = getConnection(getTestName());
+        Statement stat = conn.createStatement();
+        stat.execute("create table test(id identity primary key, lob clob)");
+        conn.close();
+        byte[] buffer = new byte[8192];
+        for (int i = 0; i < 20; i++) {
+            conn = getConnection(getTestName());
+            stat = conn.createStatement();
+            stat.execute("insert into test(lob) select space(1025) from system_range(1, 10)");
+            stat.execute("delete from test where random() > 0.5");
+            ResultSet rs = conn.createStatement().executeQuery(
+                    "select lob from test");
+            while (rs.next()) {
+                InputStream is = rs.getBinaryStream(1);
+                while (is.read(buffer) != -1) {
+                    // ignore
+                }
+            }
+            conn.close();
+        }
+    }
+
+    private void testShutdownDuringLobCreation() throws Exception {
+        deleteDb(getTestName());
+        Connection conn = getConnection(getTestName());
+        Statement stat = conn.createStatement();
+        stat.execute("create table test(data clob) as select space(10000)");
+        final PreparedStatement prep = conn
+                .prepareStatement("set @lob = ?");
+        final AtomicBoolean end = new AtomicBoolean();
+        Task t = new Task() {
+
+            @Override
+            public void call() throws Exception {
+                prep.setBinaryStream(1, new InputStream() {
+
+                    int len;
+
+                    @Override
+                    public int read() throws IOException {
+                        if (len++ < 1024 * 1024 * 4) {
+                            return 0;
+                        }
+                        end.set(true);
+                        while (!stop) {
+                            try {
+                                Thread.sleep(1);
+                            } catch (InterruptedException e) {
+                                // ignore
+                            }
+                        }
+                        return -1;
+                    }
+                }    , -1);
+            }
+        };
+        t.execute();
+        while (!end.get()) {
+            Thread.sleep(1);
+        }
+        stat.execute("checkpoint");
+        stat.execute("shutdown immediately");
+        Exception ex = t.getException();
+        assertTrue(ex != null);
+        try {
+            conn.close();
+        } catch (Exception e) {
+            // ignore
+        }
+        conn = getConnection(getTestName());
+        stat = conn.createStatement();
+        stat.execute("shutdown defrag");
+        try {
+            conn.close();
+        } catch (Exception e) {
+            // ignore
+        }
+        conn = getConnection(getTestName());
+        stat = conn.createStatement();
+        ResultSet rs = stat.executeQuery("select * " +
+                "from information_schema.settings " +
+                "where name = 'info.PAGE_COUNT'");
+        rs.next();
+        int pages = rs.getInt(2);
+        // only one lob should remain (but it is small and compressed)
+        assertTrue("p:" + pages, pages < 4);
+        conn.close();
+    }
+
+    private void testLobCreationThenShutdown() throws Exception {
+        deleteDb(getTestName());
+        Connection conn = getConnection(getTestName());
+        Statement stat = conn.createStatement();
+        stat.execute("create table test(id identity, data clob)");
+        PreparedStatement prep = conn
+                .prepareStatement("insert into test values(?, ?)");
+        for (int i = 0; i < 9; i++) {
+            prep.setInt(1, i);
+            int size = i * i * i * i * 1024;
+            prep.setCharacterStream(2, new StringReader(new String(
+                    new char[size])));
+            prep.execute();
+        }
+        stat.execute("shutdown immediately");
+        try {
+            conn.close();
+        } catch (Exception e) {
+            // ignore
+        }
+        conn = getConnection(getTestName());
+        stat = conn.createStatement();
+        stat.execute("drop all objects");
+        stat.execute("shutdown defrag");
+        try {
+            conn.close();
+        } catch (Exception e) {
+            // ignore
+        }
+        conn = getConnection(getTestName());
+        stat = conn.createStatement();
+        ResultSet rs = stat.executeQuery("select * " +
+                "from information_schema.settings " +
+                "where name = 'info.PAGE_COUNT'");
+        rs.next();
+        int pages = rs.getInt(2);
+        // no lobs should remain
+        assertTrue("p:" + pages, pages < 4);
+        conn.close();
+    }
+
+    private void testManyTransactions() throws Exception {
+        deleteDb(getTestName());
+        Connection conn = getConnection(getTestName());
+        Statement stat = conn.createStatement();
+        stat.execute("create table test()");
+        conn.setAutoCommit(false);
+        stat.execute("insert into test values()");
+
+        Connection conn2 = getConnection(getTestName());
+        Statement stat2 = conn2.createStatement();
+        for (long i = 0; i < 100000; i++) {
+            stat2.execute("insert into test values()");
+        }
+        conn2.close();
+        conn.close();
+    }
+
     private void testAppendOnly() throws Exception {
-        deleteDb("testAppendOnly");
-        Connection conn = getConnection(
-                "testAppendOnly");
+        deleteDb(getTestName());
+        Connection conn = getConnection(getTestName());
         Statement stat = conn.createStatement();
         stat.execute("set retention_time 0");
         for (int i = 0; i < 10; i++) {
@@ -95,11 +270,11 @@ public class TestMVTableEngine extends TestBase {
         }
         stat.execute("create table test as select x from system_range(1, 1000)");
         conn.close();
-        String fileName = getBaseDir() + "/testAppendOnly" + Constants.SUFFIX_MV_FILE;
+        String fileName = getBaseDir() + "/" + getTestName() + Constants.SUFFIX_MV_FILE;
         long fileSize = FileUtils.size(fileName);
 
         conn = getConnection(
-                "testAppendOnly;reuse_space=false");
+                getTestName() + ";reuse_space=false");
         stat = conn.createStatement();
         stat.execute("set retention_time 0");
         for (int i = 0; i < 10; i++) {
@@ -114,9 +289,9 @@ public class TestMVTableEngine extends TestBase {
         FileChannel fc = FileUtils.open(fileName, "rw");
         // undo all changes
         fc.truncate(fileSize);
+        fc.close();
 
-        conn = getConnection(
-                "testAppendOnly");
+        conn = getConnection(getTestName());
         stat = conn.createStatement();
         stat.execute("select * from dummy0 where 1 = 0");
         stat.execute("select * from dummy9 where 1 = 0");
@@ -125,11 +300,11 @@ public class TestMVTableEngine extends TestBase {
     }
 
     private void testLowRetentionTime() throws SQLException {
-        deleteDb("testLowRetentionTime");
+        deleteDb(getTestName());
         Connection conn = getConnection(
-                "testLowRetentionTime;RETENTION_TIME=10;WRITE_DELAY=10");
+                getTestName() + ";RETENTION_TIME=10;WRITE_DELAY=10");
         Statement stat = conn.createStatement();
-        Connection conn2 = getConnection("testLowRetentionTime");
+        Connection conn2 = getConnection(getTestName());
         Statement stat2 = conn2.createStatement();
         stat.execute("create alias sleep as " +
                 "$$void sleep(int ms) throws Exception { Thread.sleep(ms); }$$");
@@ -154,12 +329,11 @@ public class TestMVTableEngine extends TestBase {
     }
 
     private void testOldAndNew() throws SQLException {
-        FileUtils.deleteRecursive(getBaseDir(), true);
         Connection conn;
-
-        String urlOld = getURL("mvstore;MV_STORE=FALSE", true);
-        String urlNew = getURL("mvstore;MV_STORE=TRUE", true);
-        String url = getURL("mvstore", true);
+        deleteDb(getTestName());
+        String urlOld = getURL(getTestName() + ";MV_STORE=FALSE", true);
+        String urlNew = getURL(getTestName() + ";MV_STORE=TRUE", true);
+        String url = getURL(getTestName(), true);
 
         conn = getConnection(urlOld);
         conn.createStatement().execute("create table test_old(id int)");
@@ -182,10 +356,10 @@ public class TestMVTableEngine extends TestBase {
     }
 
     private void testTemporaryTables() throws SQLException {
-        FileUtils.deleteRecursive(getBaseDir(), true);
         Connection conn;
         Statement stat;
-        String url = "mvstore;MV_STORE=TRUE";
+        deleteDb(getTestName());
+        String url = getTestName() + ";MV_STORE=TRUE";
         url = getURL(url, true);
         conn = getConnection(url);
         stat = conn.createStatement();
@@ -210,10 +384,10 @@ public class TestMVTableEngine extends TestBase {
     }
 
     private void testUniqueIndex() throws SQLException {
-        FileUtils.deleteRecursive(getBaseDir(), true);
         Connection conn;
         Statement stat;
-        String url = "mvstore;MV_STORE=TRUE";
+        deleteDb(getTestName());
+        String url = getTestName() + ";MV_STORE=TRUE";
         url = getURL(url, true);
         conn = getConnection(url);
         stat = conn.createStatement();
@@ -226,10 +400,10 @@ public class TestMVTableEngine extends TestBase {
     }
 
     private void testSecondaryIndex() throws SQLException {
-        FileUtils.deleteRecursive(getBaseDir(), true);
         Connection conn;
         Statement stat;
-        String url = "mvstore;MV_STORE=TRUE";
+        deleteDb(getTestName());
+        String url = getTestName() + ";MV_STORE=TRUE";
         url = getURL(url, true);
         conn = getConnection(url);
         stat = conn.createStatement();
@@ -248,10 +422,10 @@ public class TestMVTableEngine extends TestBase {
     }
 
     private void testGarbageCollectionForLOB() throws SQLException {
-        FileUtils.deleteRecursive(getBaseDir(), true);
         Connection conn;
         Statement stat;
-        String url = "mvstore;MV_STORE=TRUE";
+        deleteDb(getTestName());
+        String url = getTestName() + ";MV_STORE=TRUE";
         url = getURL(url, true);
         conn = getConnection(url);
         stat = conn.createStatement();
@@ -270,7 +444,7 @@ public class TestMVTableEngine extends TestBase {
         assertThrows(ErrorCode.IO_EXCEPTION_1, prep).
             setBinaryStream(1, createFailingStream(new IllegalStateException()));
         conn.close();
-        MVStore s = MVStore.open(getBaseDir()+ "/mvstore.mv.db");
+        MVStore s = MVStore.open(getBaseDir()+ "/" + getTestName() + ".mv.db");
         assertTrue(s.hasMap("lobData"));
         MVMap<Long, byte[]> lobData = s.openMap("lobData");
         assertEquals(0, lobData.sizeAsLong());
@@ -284,10 +458,10 @@ public class TestMVTableEngine extends TestBase {
     }
 
     private void testSpatial() throws SQLException {
-        FileUtils.deleteRecursive(getBaseDir(), true);
         Connection conn;
         Statement stat;
-        String url = "mvstore;MV_STORE=TRUE";
+        deleteDb(getTestName());
+        String url = getTestName() + ";MV_STORE=TRUE";
         url = getURL(url, true);
         conn = getConnection(url);
         stat = conn.createStatement();
@@ -307,12 +481,12 @@ public class TestMVTableEngine extends TestBase {
             return;
         }
 
-        FileUtils.deleteRecursive(getBaseDir(), true);
         Connection conn;
         Connection conn2;
         Statement stat;
         Statement stat2;
-        String url = "mvstore;MV_STORE=TRUE;MVCC=TRUE";
+        deleteDb(getTestName());
+        String url = getTestName() + ";MV_STORE=TRUE;MVCC=TRUE";
         url = getURL(url, true);
         conn = getConnection(url);
         stat = conn.createStatement();
@@ -364,12 +538,12 @@ public class TestMVTableEngine extends TestBase {
     }
 
     private void testMinMaxWithNull() throws Exception {
-        FileUtils.deleteRecursive(getBaseDir(), true);
         Connection conn;
         Connection conn2;
         Statement stat;
         Statement stat2;
-        String url = "mvstore;MV_STORE=TRUE;MVCC=TRUE";
+        deleteDb(getTestName());
+        String url = getTestName() + ";MV_STORE=TRUE;MVCC=TRUE";
         url = getURL(url, true);
         conn = getConnection(url);
         stat = conn.createStatement();
@@ -394,12 +568,12 @@ public class TestMVTableEngine extends TestBase {
     }
 
     private void testTimeout() throws Exception {
-        FileUtils.deleteRecursive(getBaseDir(), true);
         Connection conn;
         Connection conn2;
         Statement stat;
         Statement stat2;
-        String url = "mvstore;MV_STORE=TRUE;MVCC=TRUE";
+        deleteDb(getTestName());
+        String url = getTestName() + ";MV_STORE=TRUE;MVCC=TRUE";
         url = getURL(url, true);
         conn = getConnection(url);
         stat = conn.createStatement();
@@ -419,10 +593,10 @@ public class TestMVTableEngine extends TestBase {
         if (config.memory) {
             return;
         }
-        FileUtils.deleteRecursive(getBaseDir(), true);
         Connection conn;
         Statement stat;
-        String url = "mvstore;MV_STORE=TRUE";
+        deleteDb(getTestName());
+        String url = getTestName() + ";MV_STORE=TRUE";
         url = getURL(url, true);
         conn = getConnection(url);
         stat = conn.createStatement();
@@ -444,39 +618,46 @@ public class TestMVTableEngine extends TestBase {
     }
 
     private void testTransactionLogUsuallyNotStored() throws Exception {
-        FileUtils.deleteRecursive(getBaseDir(), true);
         Connection conn;
         Statement stat;
-        String url = "mvstore;MV_STORE=TRUE";
-        url = getURL(url, true);
-        conn = getConnection(url);
-        stat = conn.createStatement();
-        stat.execute("create table test(id identity, name varchar)");
-        conn.setAutoCommit(false);
-        PreparedStatement prep = conn.prepareStatement(
-                "insert into test(name) values(space(10000))");
-        for (int j = 0; j < 100; j++) {
-            for (int i = 0; i < 100; i++) {
-                prep.execute();
+        // we expect the transaction log is empty in at least some of the cases
+        for (int test = 0; test < 5; test++) {
+            deleteDb(getTestName());
+            String url = getTestName() + ";MV_STORE=TRUE";
+            url = getURL(url, true);
+            conn = getConnection(url);
+            stat = conn.createStatement();
+            stat.execute("create table test(id identity, name varchar)");
+            conn.setAutoCommit(false);
+            PreparedStatement prep = conn.prepareStatement(
+                    "insert into test(name) values(space(10000))");
+            for (int j = 0; j < 100; j++) {
+                for (int i = 0; i < 100; i++) {
+                    prep.execute();
+                }
+                conn.commit();
+            }
+            stat.execute("shutdown immediately");
+            JdbcUtils.closeSilently(conn);
+
+            String file = getBaseDir() + "/" + getTestName() +
+                    Constants.SUFFIX_MV_FILE;
+
+            MVStore store = MVStore.open(file);
+            TransactionStore t = new TransactionStore(store);
+            t.init();
+            int openTransactions = t.getOpenTransactions().size();
+            store.close();
+            if (openTransactions == 0) {
+                return;
             }
-            conn.commit();
         }
-        stat.execute("shutdown immediately");
-        JdbcUtils.closeSilently(conn);
-
-        String file = getBaseDir() + "/mvstore" + Constants.SUFFIX_MV_FILE;
-
-        MVStore store = MVStore.open(file);
-        TransactionStore t = new TransactionStore(store);
-        t.init();
-        assertEquals(0, t.getOpenTransactions().size());
-        store.close();
+        fail("transaction log was never empty");
     }
 
     private void testShrinkDatabaseFile() throws Exception {
-        FileUtils.deleteRecursive(getBaseDir(), true);
-        String dbName = "mvstore" +
-                ";MV_STORE=TRUE";
+        deleteDb(getTestName());
+        String dbName = getTestName() + ";MV_STORE=TRUE";
         Connection conn;
         Statement stat;
         long maxSize = 0;
@@ -517,7 +698,7 @@ public class TestMVTableEngine extends TestBase {
             } catch (Exception e) {
                 // ignore
             }
-            String fileName = getBaseDir() + "/mvstore"
+            String fileName = getBaseDir() + "/" + getTestName()
                     + Constants.SUFFIX_MV_FILE;
             long size = FileUtils.size(fileName);
             if (i < 10) {
@@ -526,22 +707,22 @@ public class TestMVTableEngine extends TestBase {
                 fail(i + " size: " + size + " max: " + maxSize);
             }
         }
-        long sizeOld = FileUtils.size(getBaseDir() + "/mvstore"
+        long sizeOld = FileUtils.size(getBaseDir() + "/" + getTestName()
                 + Constants.SUFFIX_MV_FILE);
         conn = getConnection(dbName);
         stat = conn.createStatement();
         stat.execute("shutdown compact");
         conn.close();
-        long sizeNew = FileUtils.size(getBaseDir() + "/mvstore"
+        long sizeNew = FileUtils.size(getBaseDir() + "/" + getTestName()
                 + Constants.SUFFIX_MV_FILE);
         assertTrue("new: " + sizeNew + " old: " + sizeOld, sizeNew < sizeOld);
     }
 
     private void testTwoPhaseCommit() throws Exception {
-        FileUtils.deleteRecursive(getBaseDir(), true);
         Connection conn;
         Statement stat;
-        String url = "mvstore;MV_STORE=TRUE";
+        deleteDb(getTestName());
+        String url = getTestName() + ";MV_STORE=TRUE";
         url = getURL(url, true);
         conn = getConnection(url);
         stat = conn.createStatement();
@@ -565,10 +746,10 @@ public class TestMVTableEngine extends TestBase {
     }
 
     private void testRecover() throws Exception {
-        FileUtils.deleteRecursive(getBaseDir(), true);
         Connection conn;
         Statement stat;
-        String url = "mvstore;MV_STORE=TRUE";
+        deleteDb(getTestName());
+        String url = getTestName() + ";MV_STORE=TRUE";
         url = getURL(url, true);
         conn = getConnection(url);
         stat = conn.createStatement();
@@ -578,11 +759,11 @@ public class TestMVTableEngine extends TestBase {
         stat.execute("insert into test2 values('Hello World')");
         conn.close();
 
-        Recover.execute(getBaseDir(), "mvstore");
-        DeleteDbFiles.execute(getBaseDir(), "mvstore", true);
+        Recover.execute(getBaseDir(), getTestName());
+        deleteDb(getTestName());
         conn = getConnection(url);
         stat = conn.createStatement();
-        stat.execute("runscript from '" + getBaseDir() + "/mvstore.h2.sql'");
+        stat.execute("runscript from '" + getBaseDir() + "/" + getTestName()+ ".h2.sql'");
         ResultSet rs;
         rs = stat.executeQuery("select * from test");
         assertTrue(rs.next());
@@ -595,10 +776,10 @@ public class TestMVTableEngine extends TestBase {
     }
 
     private void testRollback() throws Exception {
-        FileUtils.deleteRecursive(getBaseDir(), true);
         Connection conn;
         Statement stat;
-        String url = "mvstore;MV_STORE=TRUE";
+        deleteDb(getTestName());
+        String url = getTestName() + ";MV_STORE=TRUE";
         conn = getConnection(url);
         stat = conn.createStatement();
         stat.execute("create table test(id identity)");
@@ -610,11 +791,10 @@ public class TestMVTableEngine extends TestBase {
     }
 
     private void testSeparateKey() throws Exception {
-        FileUtils.deleteRecursive(getBaseDir(), true);
         Connection conn;
         Statement stat;
-
-        String url = "mvstore;MV_STORE=TRUE";
+        deleteDb(getTestName());
+        String url = getTestName() + ";MV_STORE=TRUE";
 
         conn = getConnection(url);
         stat = conn.createStatement();
@@ -636,12 +816,11 @@ public class TestMVTableEngine extends TestBase {
     }
 
     private void testRollbackAfterCrash() throws Exception {
-        FileUtils.deleteRecursive(getBaseDir(), true);
         Connection conn;
         Statement stat;
-
-        String url = "mvstore;MV_STORE=TRUE";
-        String url2 = "mvstore2;MV_STORE=TRUE";
+        deleteDb(getTestName());
+        String url = getTestName() + ";MV_STORE=TRUE";
+        String url2 = getTestName() + "2;MV_STORE=TRUE";
 
         conn = getConnection(url);
         stat = conn.createStatement();
@@ -684,9 +863,10 @@ public class TestMVTableEngine extends TestBase {
                 "from system_range(1, 10)");
         conn.setAutoCommit(false);
         stat.execute("delete from test where id > 5");
-        stat.execute("backup to '" + getBaseDir() + "/backup.zip'");
+        stat.execute("backup to '" + getBaseDir() + "/" + getTestName() + ".zip'");
         conn.rollback();
-        Restore.execute(getBaseDir() + "/backup.zip", getBaseDir(), "mvstore2");
+        Restore.execute(getBaseDir() + "/" +getTestName() + ".zip",
+                getBaseDir(), getTestName() + "2");
         Connection conn2;
         conn2 = getConnection(url2);
         conn.close();
@@ -695,11 +875,10 @@ public class TestMVTableEngine extends TestBase {
     }
 
     private void testReferentialIntegrity() throws Exception {
-        FileUtils.deleteRecursive(getBaseDir(), true);
         Connection conn;
         Statement stat;
-
-        conn = getConnection("mvstore;MV_STORE=TRUE");
+        deleteDb(getTestName());
+        conn = getConnection(getTestName() + ";MV_STORE=TRUE");
 
         stat = conn.createStatement();
         stat.execute("create table test(id int, parent int " +
@@ -768,11 +947,11 @@ public class TestMVTableEngine extends TestBase {
     }
 
     private void testWriteDelay() throws Exception {
-        FileUtils.deleteRecursive(getBaseDir(), true);
         Connection conn;
         Statement stat;
         ResultSet rs;
-        conn = getConnection("mvstore;MV_STORE=TRUE");
+        deleteDb(getTestName());
+        conn = getConnection(getTestName() + ";MV_STORE=TRUE");
         stat = conn.createStatement();
         stat.execute("create table test(id int)");
         stat.execute("set write_delay 0");
@@ -783,7 +962,7 @@ public class TestMVTableEngine extends TestBase {
         } catch (Exception e) {
             // ignore
         }
-        conn = getConnection("mvstore;MV_STORE=TRUE");
+        conn = getConnection(getTestName() + ";MV_STORE=TRUE");
         stat = conn.createStatement();
         rs = stat.executeQuery("select * from test");
         assertTrue(rs.next());
@@ -791,11 +970,11 @@ public class TestMVTableEngine extends TestBase {
     }
 
     private void testAutoCommit() throws SQLException {
-        FileUtils.deleteRecursive(getBaseDir(), true);
         Connection conn;
         Statement stat;
         ResultSet rs;
-        conn = getConnection("mvstore;MV_STORE=TRUE");
+        deleteDb(getTestName());
+        conn = getConnection(getTestName() + ";MV_STORE=TRUE");
         for (int i = 0; i < 2; i++) {
             stat = conn.createStatement();
             stat.execute("create table test(id int primary key, name varchar)");
@@ -825,22 +1004,22 @@ public class TestMVTableEngine extends TestBase {
     }
 
     private void testReopen() throws SQLException {
-        FileUtils.deleteRecursive(getBaseDir(), true);
         Connection conn;
         Statement stat;
-        conn = getConnection("mvstore;MV_STORE=TRUE");
+        deleteDb(getTestName());
+        conn = getConnection(getTestName() + ";MV_STORE=TRUE");
         stat = conn.createStatement();
         stat.execute("create table test(id int, name varchar)");
         conn.close();
-        conn = getConnection("mvstore;MV_STORE=TRUE");
+        conn = getConnection(getTestName() + ";MV_STORE=TRUE");
         stat = conn.createStatement();
         stat.execute("drop table test");
         conn.close();
     }
 
     private void testBlob() throws SQLException, IOException {
-        FileUtils.deleteRecursive(getBaseDir(), true);
-        String dbName = "mvstore;MV_STORE=TRUE";
+        deleteDb(getTestName());
+        String dbName = getTestName() + ";MV_STORE=TRUE";
         Connection conn;
         Statement stat;
         conn = getConnection(dbName);
@@ -863,12 +1042,11 @@ public class TestMVTableEngine extends TestBase {
             assertEquals(129, len);
         }
         conn.close();
-        FileUtils.deleteRecursive(getBaseDir(), true);
     }
 
     private void testEncryption() throws Exception {
-        FileUtils.deleteRecursive(getBaseDir(), true);
-        String dbName = "mvstore;MV_STORE=TRUE";
+        deleteDb(getTestName());
+        String dbName = getTestName() + ";MV_STORE=TRUE";
         Connection conn;
         Statement stat;
         String url = getURL(dbName + ";CIPHER=AES", true);
@@ -883,12 +1061,11 @@ public class TestMVTableEngine extends TestBase {
         stat.execute("select * from test");
         stat.execute("drop table test");
         conn.close();
-        FileUtils.deleteRecursive(getBaseDir(), true);
     }
 
     private void testExclusiveLock() throws Exception {
-        FileUtils.deleteRecursive(getBaseDir(), true);
-        String dbName = "mvstore;MV_STORE=TRUE;MVCC=FALSE";
+        deleteDb(getTestName());
+        String dbName = getTestName() + ";MV_STORE=TRUE;MVCC=FALSE";
         Connection conn, conn2;
         Statement stat, stat2;
         conn = getConnection(dbName);
@@ -907,31 +1084,29 @@ public class TestMVTableEngine extends TestBase {
         assertEquals("WRITE", rs2.getString("lock_type"));
         conn2.close();
         conn.close();
-        FileUtils.deleteRecursive(getBaseDir(), true);
     }
 
     private void testReadOnly() throws Exception {
-        FileUtils.deleteRecursive(getBaseDir(), true);
-        String dbName = "mvstore;MV_STORE=TRUE";
+        deleteDb(getTestName());
+        String dbName = getTestName() + ";MV_STORE=TRUE";
         Connection conn;
         Statement stat;
         conn = getConnection(dbName);
         stat = conn.createStatement();
         stat.execute("create table test(id int)");
         conn.close();
-        FileUtils.setReadOnly(getBaseDir() + "/mvstore" +
+        FileUtils.setReadOnly(getBaseDir() + "/" + getTestName() +
                 Constants.SUFFIX_MV_FILE);
         conn = getConnection(dbName);
         Database db = (Database) ((JdbcConnection) conn).getSession()
                 .getDataHandler();
         assertTrue(db.getMvStore().getStore().getFileStore().isReadOnly());
         conn.close();
-        FileUtils.deleteRecursive(getBaseDir(), true);
     }
 
     private void testReuseDiskSpace() throws Exception {
-        FileUtils.deleteRecursive(getBaseDir(), true);
-        String dbName = "mvstore;MV_STORE=TRUE";
+        deleteDb(getTestName());
+        String dbName = getTestName() + ";MV_STORE=TRUE";
         Connection conn;
         Statement stat;
         long maxSize = 0;
@@ -946,7 +1121,7 @@ public class TestMVTableEngine extends TestBase {
                     "from system_range(1, 1000)");
             stat.execute("drop table test");
             conn.close();
-            long size = FileUtils.size(getBaseDir() + "/mvstore"
+            long size = FileUtils.size(getBaseDir() + "/" + getTestName()
                     + Constants.SUFFIX_MV_FILE);
             if (i < 10) {
                 maxSize = (int) (Math.max(size, maxSize) * 1.1);
@@ -957,8 +1132,8 @@ public class TestMVTableEngine extends TestBase {
     }
 
     private void testDataTypes() throws Exception {
-        FileUtils.deleteRecursive(getBaseDir(), true);
-        String dbName = "mvstore;MV_STORE=TRUE";
+        deleteDb(getTestName());
+        String dbName = getTestName() + ";MV_STORE=TRUE";
         Connection conn = getConnection(dbName);
         Statement stat = conn.createStatement();
 
@@ -1117,8 +1292,8 @@ public class TestMVTableEngine extends TestBase {
     }
 
     private void testLocking() throws Exception {
-        FileUtils.deleteRecursive(getBaseDir(), true);
-        String dbName = "mvstore;MV_STORE=TRUE;MVCC=FALSE";
+        deleteDb(getTestName());
+        String dbName = getTestName() + ";MV_STORE=TRUE;MVCC=FALSE";
         Connection conn = getConnection(dbName);
         Statement stat = conn.createStatement();
         stat.execute("set lock_timeout 1000");
@@ -1153,8 +1328,8 @@ public class TestMVTableEngine extends TestBase {
     }
 
     private void testSimple() throws Exception {
-        FileUtils.deleteRecursive(getBaseDir(), true);
-        String dbName = "mvstore;MV_STORE=TRUE";
+        deleteDb(getTestName());
+        String dbName = getTestName() + ";MV_STORE=TRUE";
         Connection conn = getConnection(dbName);
         Statement stat = conn.createStatement();
         stat.execute("create table test(id int primary key, name varchar)");
